searchData={"items":[{"type":"module","title":"MDEx","doc":"Fast and Extensible Markdown for Elixir.","ref":"MDEx.html"},{"type":"module","title":"Features - MDEx","doc":"- Support formats:\n  - Markdown (CommonMark)\n  - HTML\n  - JSON\n  - XML\n- Floki-like [Document AST](https://hexdocs.pm/mdex/MDEx.Document.html)\n- Req-like [Pipeline API](https://hexdocs.pm/mdex/MDEx.Pipe.html)\n- Compliant with the [CommonMark spec](https://commonmark.org)\n- Additional features:\n  - GitHub Flavored Markdown\n  - Discord and GitLab features\n  - Wiki-style links\n  - Emoji shortcodes\n  - [Syntax highlighting](https://autumnus.dev) for code blocks\n  - HTML sanitization\n  - ~MD sigil for Markdown, HTML, JSON, and XML","ref":"MDEx.html#module-features"},{"type":"module","title":"Foundation - MDEx","doc":"The library is built on top of:\n\n- [comrak](https://crates.io/crates/comrak) - a fast Rust port of [GitHub's CommonMark parser](https://github.com/github/cmark-gfm)\n- [ammonia](https://crates.io/crates/ammonia) for HTML sanitization\n- [autumnus](https://crates.io/crates/autumnus) for syntax highlighting","ref":"MDEx.html#module-foundation"},{"type":"module","title":"Installation - MDEx","doc":"Add `:mdex` dependency:\n\n```elixir\ndef deps do\n  [\n    {:mdex, \"~> 0.7\"}\n  ]\nend\n```","ref":"MDEx.html#module-installation"},{"type":"module","title":"Usage - MDEx","doc":"```elixir\nMix.install([{:mdex, \"~> 0.7\"}])\n```\n\n```elixir\niex> MDEx.to_html!(\"# Hello\")\n\" Hello \"\n```\n\n```elixir\niex> MDEx.to_html!(\"# Hello :smile:\", extension: [shortcodes: true])\n\" Hello 😄 \"\n```","ref":"MDEx.html#module-usage"},{"type":"module","title":"Plugins - MDEx","doc":"- [mdex_mermaid](https://hex.pm/packages/mdex_mermaid) - Render [Mermaid](https://mermaid.js.org) diagrams in code blocks","ref":"MDEx.html#module-plugins"},{"type":"module","title":"Req-like Pipeline - MDEx","doc":"[MDEx.Pipe](https://hexdocs.pm/mdex/MDEx.Pipe.html) provides a high-level API to manipulate a Markdown document and build plugins that can be attached to a pipeline:\n\n```elixir\ndocument = ~s|\n# Super Diagram\n\n\\`\\`\\`mermaid\ngraph TD\n  A[Enter Chart Definition] --> B(Preview)\n  B --> C{decide}\n  C --> D[Keep]\n  C --> E[Edit Definition]\n  E --> B\n  D --> F[Save Image and Code]\n  F --> B\n\\`\\`\\`\n|\n\nMDEx.new()\n|> MDExMermaid.attach(mermaid_version: \"11\")\n|> MDEx.to_html(document: document)\n```\n\n## ~MD Sigil\n\nConvert and generate AST (MDEx.Document), Markdown (CommonMark), HTML, JSON, and XML formats.\n\n```elixir\niex> import MDEx.Sigil\niex> ~MD|# Hello from `~MD` sigil|\n%MDEx.Document{\n  nodes: [\n    %MDEx.Heading{\n      nodes: [\n        %MDEx.Text{literal: \"Hello from \"},\n        %MDEx.Code{num_backticks: 1, literal: \"~MD\"},\n        %MDEx.Text{literal: \" sigil\"}\n      ],\n      level: 1,\n      setext: false\n    }\n  ]\n}\n```\n\n```elixir\niex> import MDEx.Sigil\niex> ~MD|`~MD` also converts to HTML format|HTML\n\"  ~MD  also converts to HTML format \"\n```\n\n```elixir\niex> import MDEx.Sigil\niex> ~MD|and to XML as well|XML\n\"<?xml version=\\\"1.0\\\" encoding=\\\"UTF-8\\\"?>\\n<!DOCTYPE document SYSTEM \\\"CommonMark.dtd\\\">\\n \\n   \\n     and to XML as well \\n   \\n \"\n```\n\n`~MD` also accepts an `assigns` map to pass variables to the document:\n\n```elixir\niex> import MDEx.Sigil\niex> assigns = %{lang: \"Elixir\"}\niex> ~MD|Running <%= @lang %>|HTML\n\" Running Elixir \"\n```\n\nSee more info at https://hexdocs.pm/mdex/MDEx.Sigil.html","ref":"MDEx.html#module-req-like-pipeline"},{"type":"module","title":"Safety - MDEx","doc":"For security reasons, every piece of raw HTML is omitted from the output by default:\n\n```elixir\niex> MDEx.to_html!(\" Hello \")\n\"<!-- raw HTML omitted -->\"\n```\n\nThat's not very useful for most cases, but you have a few options:","ref":"MDEx.html#module-safety"},{"type":"module","title":"Escape - MDEx","doc":"The most basic is render raw HTML but escape it:\n\n```elixir\niex> MDEx.to_html!(\" Hello \", render: [escape: true])\n\"&lt;h1&gt;Hello&lt;/h1&gt;\"\n```","ref":"MDEx.html#module-escape"},{"type":"module","title":"Sanitize - MDEx","doc":"But if the input is provided by external sources, it might be a good idea to sanitize it:\n\n```elixir\niex> MDEx.to_html!(\" Elixir \", render: [unsafe: true], sanitize: MDEx.default_sanitize_options())\n\"  Elixir  \"\n```\n\nNote that you must pass the `unsafe: true` option to first generate the raw HTML in order to sanitize it.\n\nIt does clean HTML with a [conservative set of defaults](https://docs.rs/ammonia/latest/ammonia/fn.clean.html)\nthat works for most cases, but you can overwrite those rules for further customization.\n\nFor example, let's modify the [link rel](https://docs.rs/ammonia/latest/ammonia/struct.Builder.html#method.link_rel) attribute\nto add `\"nofollow\"` into the `rel` attribute:\n\n```elixir\niex> MDEx.to_html!(\" External \", render: [unsafe: true], sanitize: [link_rel: \"nofollow noopener noreferrer\"])\n\"  External  \"\n```\n\nIn this case the default rule set is still applied but the `link_rel` rule is overwritten.","ref":"MDEx.html#module-sanitize"},{"type":"module","title":"Unsafe - MDEx","doc":"If those rules are too strict and you really trust the input, or you really need to render raw HTML,\nthen you can just render it directly without escaping nor sanitizing:\n\n```elixir\niex> MDEx.to_html!(\" alert('hello') \", render: [unsafe: true])\n\" alert('hello') \"\n```","ref":"MDEx.html#module-unsafe"},{"type":"module","title":"Parsing - MDEx","doc":"Converts Markdown to an AST data structure that can be inspected and manipulated to change the content of the document programmatically.\n\nThe data structure format is inspired on [Floki](https://github.com/philss/floki) (with `:attributes_as_maps = true`) so we can keep similar APIs and keep the same mental model when\nworking with these documents, either Markdown or HTML, where each node is represented as a struct holding the node name as the struct name and its attributes and children, for eg:\n\n```elixir\n%MDEx.Heading{\n  level: 1\n  nodes: [...],\n}\n```\n\nThe parent node that represents the root of the document is the [MDEx.Document](https://hexdocs.pm/mdex/MDEx.Document.html) struct,\nwhere you can find more more information about the AST and what operations are available.\n\nThe complete list of nodes is listed in the [documentation](https://hexdocs.pm/mdex/), section `Document Nodes`.","ref":"MDEx.html#module-parsing"},{"type":"module","title":"Formatting - MDEx","doc":"Formatting is the process of converting from one format to another, for example from AST or Markdown to HTML.\nFormatting to XML and to Markdown is also supported.\n\nYou can use [MDEx.parse_document/2](https://hexdocs.pm/mdex/MDEx.html#parse_document/2) to generate an AST or any of the `to_*` functions\nto convert to Markdown (CommonMark), HTML, JSON, or XML.","ref":"MDEx.html#module-formatting"},{"type":"module","title":"Examples - MDEx","doc":"","ref":"MDEx.html#module-examples"},{"type":"module","title":"GitHub Flavored Markdown with emojis - MDEx","doc":"```elixir\nMDEx.to_html!(~S\"\"\"\n# GitHub Flavored Markdown :rocket:\n\n- [x] Task A\n- [x] Task B\n- [ ] Task C\n\n| Feature | Status |\n| ------- | ------ |\n| Fast | :white_check_mark: |\n| GFM  | :white_check_mark: |\n\nCheck out the spec at https://github.github.com/gfm/\n\"\"\",\nextension: [\n  strikethrough: true,\n  tagfilter: true,\n  table: true,\n  autolink: true,\n  tasklist: true,\n  footnotes: true,\n  shortcodes: true,\n],\nparse: [\n  smart: true,\n  relaxed_tasklist_matching: true,\n  relaxed_autolinks: true\n],\nrender: [\n  github_pre_lang: true,\n  unsafe: true,\n]) |> IO.puts()\n\"\"\"\n GitHub Flavored Markdown 🚀 \n \n     Task A \n     Task B \n     Task C \n \n \n   \n     \n       Feature \n       Status \n     \n   \n   \n     \n       Fast \n       ✅ \n     \n     \n       GFM \n       ✅ \n     \n   \n \n Check out the spec at  https://github.github.com/gfm/  \n\"\"\"\n```","ref":"MDEx.html#module-github-flavored-markdown-with-emojis"},{"type":"module","title":"Code Syntax Highlighting - MDEx","doc":"````elixir\nMDEx.to_html!(~S\"\"\"\n```elixir\nString.upcase(\"elixir\")\n```\n\"\"\",\nsyntax_highlight: [\n  formatter: {:html_inline, theme: \"catppuccin_latte\"}\n]\n) |> IO.puts()\n\"\"\"\n \n   \n     String  .  upcase  (  &quot;elixir&quot;  ) \n   \n \n\"\"\"\n````","ref":"MDEx.html#module-code-syntax-highlighting"},{"type":"module","title":"Pre-compilation - MDEx","doc":"Pre-compiled binaries are available for the following targets, so you don't need to have Rust installed to compile and use this library:\n\n- `aarch64-apple-darwin`\n- `aarch64-unknown-linux-gnu`\n- `aarch64-unknown-linux-musl`\n- `arm-unknown-linux-gnueabihf`\n- `riscv64gc-unknown-linux-gnu`\n- `x86_64-apple-darwin`\n- `x86_64-pc-windows-gnu`\n- `x86_64-pc-windows-msvc`\n- `x86_64-unknown-freebsd`\n- `x86_64-unknown-linux-gnu`\n- `x86_64-unknown-linux-musl`\n\n**Note:** The pre-compiled binaries for Linux are compiled using Ubuntu 22 on libc 2.35, which requires minimum Ubuntu 22, Debian Bookworm or a system with a compatible libc version. For older Linux systems, you'll need to compile manually.","ref":"MDEx.html#module-pre-compilation"},{"type":"module","title":"Compile manually - MDEx","doc":"But in case you need or want to compile it yourself, you can do the following:\n\n1. [Install Rust](https://www.rust-lang.org/tools/install)\n\n2. Install a C compiler or build packages\n\nIt depends on your OS, for example in Ubuntu you can install the `build-essential` package.\n\n3. Run:\n\n```sh\nexport MDEX_BUILD=1\nmix deps.get\nmix compile\n```","ref":"MDEx.html#module-compile-manually"},{"type":"module","title":"Legacy CPUs - MDEx","doc":"Modern CPU features are enabled by default but if your environment has an older CPU,\nyou can use legacy artifacts by adding the following configuration to your `config.exs`:\n\n```elixir\nconfig :mdex, use_legacy_artifacts: true\n```","ref":"MDEx.html#module-legacy-cpus"},{"type":"module","title":"Used By - MDEx","doc":"- [BeaconCMS](https://github.com/BeaconCMS/beacon)\n- [Tableau](https://github.com/elixir-tools/tableau)\n- [Bonfire](https://github.com/bonfire-networks/bonfire-app)\n- [00](https://github.com/technomancy-dev/00)\n- [Plural Console](https://github.com/pluralsh/console)\n- [Exmeralda](https://github.com/bitcrowd/exmeralda)\n- [Algora](https://github.com/algora-io/algora)\n- [Ash AI](https://github.com/ash-project/ash_ai)\n- [Canada Navigator](https://github.com/canada-ca/navigator)\n- And [more...](https://github.com/search?q=lang%3Aelixir+%3Amdex&type=code)\n\n_Are you using MDEx and want to list your project here? Please send a PR!_","ref":"MDEx.html#module-used-by"},{"type":"module","title":"Motivation - MDEx","doc":"MDEx was born out of the necessity of parsing CommonMark files, to parse hundreds of files quickly, and to be easily extensible by consumers of the library.\n\n- [earmark](https://hex.pm/packages/earmark) is extensible but [can't parse](https://github.com/RobertDober/earmark_parser/issues/126) all kinds of documents and is slow to convert hundreds of markdowns.\n- [md](https://hex.pm/packages/md) is very extensible but the doc says \"If one needs to perfectly parse the common markdown, Md is probably not the correct choice\" and CommonMark was a requirement to parse many existing files.\n- [markdown](https://hex.pm/packages/markdown) is not precompiled and has not received updates in a while.\n- [cmark](https://hex.pm/packages/cmark) is a fast CommonMark parser but it requires compiling the C library, is hard to extend, and was archived on Apr 2024.\n\n_Note that MDEx is the only one that syntax highlights out-of-the-box which contributes to make it slower than cmark._","ref":"MDEx.html#module-motivation"},{"type":"module","title":"Comparison - MDEx","doc":"|Feature|MDEx|Earmark|md|cmark|\n| --- | --- | --- | --- | --- |\n|Active|✅|✅|✅|❌|\n|Pure Elixir|❌|✅|✅|❌|\n|Extensible|✅|✅|✅|❌|\n|Syntax Highlighting|✅|❌|❌|❌|\n|AST|✅|✅|✅|❌|\n|AST to Markdown|✅|⚠️²|❌|❌|\n|To HTML|✅|✅|✅|✅|\n|To JSON|✅|❌|❌|❌|\n|To XML|✅|❌|❌|✅|\n|To Manpage|❌|❌|❌|✅|\n|To LaTeX|❌|❌|❌|✅|\n|Emoji|✅|❌|❌|❌|\n|GFM³|✅|✅|❌|❌|\n|GitLab⁴|⚠️¹|❌|❌|❌|\n|Discord⁵|⚠️¹|❌|❌|❌|\n\n1. Partial support\n2. Possible with [earmark_reversal](https://hex.pm/packages/earmark_reversal)\n3. GitHub Flavored Markdown\n4. GitLab Flavored Markdown\n5. Discord Flavored Markdown","ref":"MDEx.html#module-comparison"},{"type":"module","title":"Benchmark - MDEx","doc":"A [simple script](benchmark.exs) is available to compare existing libs:\n\n```\nName              ips        average  deviation         median         99th %\ncmark         22.82 K      0.0438 ms    ±16.24%      0.0429 ms      0.0598 ms\nmdex           3.57 K        0.28 ms     ±9.79%        0.28 ms        0.33 ms\nmd             0.34 K        2.95 ms    ±10.56%        2.90 ms        3.62 ms\nearmark        0.25 K        4.04 ms     ±4.50%        4.00 ms        4.44 ms\n\nComparison:\ncmark         22.82 K\nmdex           3.57 K - 6.39x slower +0.24 ms\nmd             0.34 K - 67.25x slower +2.90 ms\nearmark        0.25 K - 92.19x slower +4.00 ms\n```\n\nTo finish, a friendly reminder that all libs have their own strengths and trade-offs so use the one that better suit your needs.","ref":"MDEx.html#module-benchmark"},{"type":"module","title":"Acknowledgements - MDEx","doc":"- [comrak](https://crates.io/crates/comrak) crate for all the heavy work on parsing Markdown and rendering HTML\n- [Floki](https://hex.pm/packages/floki) for the AST\n- [Req](https://hex.pm/packages/req) for the pipeline API\n- Logo based on [markdown-mark](https://github.com/dcurtis/markdown-mark)","ref":"MDEx.html#module-acknowledgements"},{"type":"function","title":"MDEx.anchorize/1","doc":"Converts a given `text` string to a format that can be used as an \"anchor\", such as in a Table of Contents.\n\nThis uses the same algorithm GFM uses for anchor ids, so it can be used reliably.\n\n> #### Repeated anchors\n> GFM will dedupe multiple repeated anchors with the same value by appending\n> an incrementing number to the end of the anchor. That is beyond the scope of\n> this function, so you will have to handle it yourself.","ref":"MDEx.html#anchorize/1"},{"type":"function","title":"Examples - MDEx.anchorize/1","doc":"iex> MDEx.anchorize(\"Hello World\")\n    \"hello-world\"\n\n    iex> MDEx.anchorize(\"Hello, World!\")\n    \"hello-world\"\n\n    iex> MDEx.anchorize(\"Hello -- World\")\n    \"hello----world\"\n\n    iex> MDEx.anchorize(\"Hello World 123\")\n    \"hello-world-123\"\n\n    iex> MDEx.anchorize(\"你好世界\")\n    \"你好世界\"","ref":"MDEx.html#anchorize/1-examples"},{"type":"function","title":"MDEx.default_extension_options/0","doc":"Returns the default options for the `:extension` group.","ref":"MDEx.html#default_extension_options/0"},{"type":"function","title":"MDEx.default_parse_options/0","doc":"Returns the default options for the `:parse` group.","ref":"MDEx.html#default_parse_options/0"},{"type":"function","title":"MDEx.default_render_options/0","doc":"Returns the default options for the `:render` group.","ref":"MDEx.html#default_render_options/0"},{"type":"function","title":"MDEx.default_sanitize_options/0","doc":"Returns the default options for the `:sanitize` group.","ref":"MDEx.html#default_sanitize_options/0"},{"type":"function","title":"MDEx.default_syntax_highlight_options/0","doc":"Returns the default options for the `:syntax_highlight` group.","ref":"MDEx.html#default_syntax_highlight_options/0"},{"type":"type","title":"MDEx.extension_options/0","doc":"List of [comrak extension options](https://docs.rs/comrak/latest/comrak/struct.ExtensionOptions.html).","ref":"MDEx.html#t:extension_options/0"},{"type":"function","title":"MDEx.new/1","doc":"Builds a new `MDEx.Pipe` instance.\n\nOnce the pipe is complete, call either one of the following functions to format the document:\n\n- `MDEx.to_html/2`\n- `MDEx.to_json/1`\n- `MDEx.to_xml/2`\n- `MDEx.to_markdown/1`","ref":"MDEx.html#new/1"},{"type":"function","title":"Examples - MDEx.new/1","doc":"* Build a pipe with `:document`:\n\n      iex> mdex = MDEx.new(document: \"# Hello\")\n      iex> MDEx.to_html(mdex)\n      {:ok, \" Hello \"}\n\n      iex> mdex = MDEx.new(document: \"Hello ~world~\", extension: [strikethrough: true])\n      iex> MDEx.to_json(mdex)\n      {:ok, ~s|{\"nodes\":[{\"nodes\":[{\"literal\":\"Hello \",\"node_type\":\"MDEx.Text\"},{\"nodes\":[{\"literal\":\"world\",\"node_type\":\"MDEx.Text\"}],\"node_type\":\"MDEx.Strikethrough\"}],\"node_type\":\"MDEx.Paragraph\"}],\"node_type\":\"MDEx.Document\"}|}\n\n* Pass a `:document` when formatting:\n\n      iex> mdex = MDEx.new(extension: [strikethrough: true])\n      iex> MDEx.to_xml(mdex, document: \"Hello ~world~\")\n      {:ok, ~s|<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n      <!DOCTYPE document SYSTEM \"CommonMark.dtd\">\n       \n         \n           Hello  \n           \n             world \n           \n         \n       |}","ref":"MDEx.html#new/1-examples"},{"type":"function","title":"Notes - MDEx.new/1","doc":"1. Source `:document` is automatically [parsed](https://hexdocs.pm/mdex/MDEx.html#parse_document!/2)\ninto `MDEx.Document` before the pipeline runs so every step receives the same data type.\n\n2. You can pass the document when creating the pipe:\n\n```elixir\nMDEx.new(document: \"# Hello\") |> MDEx.to_html()\n```\n\nOr pass it only when formatting the document,\nuseful to reuse the same pipe with different documents and formats.\n\n```elixir\nmdex = MDEx.new()\n# ... attach plugins and steps\n\nMDEx.to_html(mdex, document: \"# Hello HTML\")\nMDEx.to_json(mdex, document: \"# Hello JSON\")\n```","ref":"MDEx.html#new/1-notes"},{"type":"type","title":"MDEx.options/0","doc":"Options to customize the parsing and rendering of Markdown documents.","ref":"MDEx.html#t:options/0"},{"type":"type","title":"Examples - MDEx.options/0","doc":"- Enable the `table` extension:\n\n    ````elixir\n    MDEx.to_html!(\"\"\"\n    | lang |\n    |------|\n    | elixir |\n    \"\"\",\n    extension: [table: true]\n    )\n    ````\n\n- Syntax highlight using inline style and the `github_light` theme:\n\n    ````elixir\n    MDEx.to_html!(\"\"\"","ref":"MDEx.html#t:options/0-examples"},{"type":"type","title":"Code Example - MDEx.options/0","doc":"```elixir\n    Atom.to_string(:elixir)\n    ```\n    \"\"\",\n    syntax_highlight: [\n      formatter: {:html_inline, theme: \"github_light\"}\n    ])\n    ````\n\n- Sanitize HTML output, in this example disallow ` ` tags:\n\n    ````elixir\n    MDEx.to_html!(\"\"\"","ref":"MDEx.html#t:options/0-code-example"},{"type":"type","title":"Links won't be displayed - MDEx.options/0","doc":"Example \n    ```\n    \"\"\",\n    sanitize: [\n      rm_tags: [\"a\"],\n    ])\n    ````","ref":"MDEx.html#t:options/0-links-won-t-be-displayed"},{"type":"type","title":"Options - MDEx.options/0","doc":"* `:document` - Markdown document, either a string or a `MDEx.Document` struct. The default value is `\"\"`.\n\n* `:extension` (`t:keyword/0`) - Enable extensions. See comrak's [ExtensionOptions](https://docs.rs/comrak/latest/comrak/struct.ExtensionOptions.html) for more info and examples. The default value is `[]`.\n\n  * `:strikethrough` (`t:boolean/0`) - Enables the [strikethrough extension](https://github.github.com/gfm/#strikethrough-extension-) from the GFM spec. The default value is `false`.\n\n  * `:tagfilter` (`t:boolean/0`) - Enables the [tagfilter extension](https://github.github.com/gfm/#disallowed-raw-html-extension-) from the GFM spec. The default value is `false`.\n\n  * `:table` (`t:boolean/0`) - Enables the [table extension](https://github.github.com/gfm/#tables-extension-) from the GFM spec. The default value is `false`.\n\n  * `:autolink` (`t:boolean/0`) - Enables the [autolink extension](https://github.github.com/gfm/#autolinks-extension-) from the GFM spec. The default value is `false`.\n\n  * `:tasklist` (`t:boolean/0`) - Enables the [task list extension](https://github.github.com/gfm/#task-list-items-extension-) from the GFM spec. The default value is `false`.\n\n  * `:superscript` (`t:boolean/0`) - Enables the superscript Comrak extension. The default value is `false`.\n\n  * `:header_ids` - Enables the header IDs Comrak extension. The default value is `nil`.\n\n  * `:footnotes` (`t:boolean/0`) - Enables the footnotes extension per cmark-gfm The default value is `false`.\n\n  * `:description_lists` (`t:boolean/0`) - Enables the description lists extension. The default value is `false`.\n\n  * `:front_matter_delimiter` - Enables the front matter extension. The default value is `nil`.\n\n  * `:multiline_block_quotes` (`t:boolean/0`) - Enables the multiline block quotes extension. The default value is `false`.\n\n  * `:alerts` (`t:boolean/0`) - Enables GitHub style alerts. The default value is `false`.\n\n  * `:math_dollars` (`t:boolean/0`) - Enables math using dollar syntax. The default value is `false`.\n\n  * `:math_code` (`t:boolean/0`) - Enables the [math code extension](https://github.github.com/gfm/#math-code) from the GFM spec. The default value is `false`.\n\n  * `:shortcodes` (`t:boolean/0`) - Phrases wrapped inside of ':' blocks will be replaced with emojis. The default value is `false`.\n\n  * `:wikilinks_title_after_pipe` (`t:boolean/0`) - Enables wikilinks using title after pipe syntax. The default value is `false`.\n\n  * `:wikilinks_title_before_pipe` (`t:boolean/0`) - Enables wikilinks using title before pipe syntax. The default value is `false`.\n\n  * `:underline` (`t:boolean/0`) - Enables underlines using double underscores. The default value is `false`.\n\n  * `:subscript` (`t:boolean/0`) - Enables subscript text using single tildes. The default value is `false`.\n\n  * `:spoiler` (`t:boolean/0`) - Enables spoilers using double vertical bars. The default value is `false`.\n\n  * `:greentext` (`t:boolean/0`) - Requires at least one space after a > character to generate a blockquote, and restarts blockquote nesting across unique lines of input. The default value is `false`.\n\n  * `:image_url_rewriter` - Wraps embedded image URLs using a string template.\n\n      Example:\n\n      Given this image `![alt text](http://unsafe.com/image.png)` and this rewriter:\n\n        image_url_rewriter: \"https://example.com?url={@url}\"\n\n      Renders `   `\n\n      Notes:\n\n      - Assign `@url` is always passed to the template.\n      - Function callback is not supported, only string templates.\n        Transform the Document AST for more complex cases.\n\n    The default value is `nil`.\n\n  * `:link_url_rewriter` - Wraps link URLs using a string template.\n\n      Example:\n\n      Given this link `[my link](http://unsafe.example.com/bad)` and this rewriter:\n\n        link_url_rewriter: \"https://safe.example.com/norefer?url={@url}\"\n\n      Renders `  my link  `\n\n      Notes:\n\n      - Assign `@url` is always passed to the template.\n      - Function callback is not supported, only string templates.\n        Transform the Document AST for more complex cases.\n\n    The default value is `nil`.\n\n* `:parse` (`t:keyword/0`) - Configure parsing behavior. See comrak's [ParseOptions](https://docs.rs/comrak/latest/comrak/struct.ParseOptions.html) for more info and examples. The default value is `[]`.\n\n  * `:smart` (`t:boolean/0`) - Punctuation (quotes, full-stops and hyphens) are converted into 'smart' punctuation. The default value is `false`.\n\n  * `:default_info_string` - The default info string for fenced code blocks. The default value is `nil`.\n\n  * `:relaxed_tasklist_matching` (`t:boolean/0`) - Whether or not a simple `x` or `X` is used for tasklist or any other symbol is allowed. The default value is `false`.\n\n  * `:relaxed_autolinks` (`t:boolean/0`) - Relax parsing of autolinks, allow links to be detected inside brackets and allow all url schemes. It is intended to allow a very specific type of autolink detection, such as `[this http://and.com that]` or `{http://foo.com}`, on a best can basis. The default value is `true`.\n\n* `:render` (`t:keyword/0`) - Configure rendering behavior. See comrak's [RenderOptions](https://docs.rs/comrak/latest/comrak/struct.RenderOptions.html) for more info and examples. The default value is `[]`.\n\n  * `:hardbreaks` (`t:boolean/0`) - [Soft line breaks](http://spec.commonmark.org/0.27/#soft-line-breaks) in the input translate into hard line breaks in the output. The default value is `false`.\n\n  * `:github_pre_lang` (`t:boolean/0`) - GitHub-style ` ` is used for fenced code blocks with info tags. The default value is `false`.\n\n  * `:full_info_string` (`t:boolean/0`) - Enable full info strings for code blocks. The default value is `false`.\n\n  * `:width` (`t:integer/0`) - The wrap column when outputting CommonMark. The default value is `0`.\n\n  * `:unsafe` (`t:boolean/0`) - Allow rendering of raw HTML and potentially dangerous links. The default value is `false`.\n\n  * `:escape` (`t:boolean/0`) - Escape raw HTML instead of clobbering it. The default value is `false`.\n\n  * `:list_style` - Set the type of [bullet list marker](https://spec.commonmark.org/0.30/#bullet-list-marker) to use.\n    Either one of `:dash`, `:plus`, or `:star`. The default value is `:dash`.\n\n  * `:sourcepos` (`t:boolean/0`) - Include source position attributes in HTML and XML output. The default value is `false`.\n\n  * `:escaped_char_spans` (`t:boolean/0`) - Wrap escaped characters in a ` ` to allow any post-processing to recognize them. The default value is `false`.\n\n  * `:ignore_setext` (`t:boolean/0`) - Ignore setext headings in input. The default value is `false`.\n\n  * `:ignore_empty_links` (`t:boolean/0`) - Ignore empty links in input. The default value is `false`.\n\n  * `:gfm_quirks` (`t:boolean/0`) - Enables GFM quirks in HTML output which break CommonMark compatibility. The default value is `false`.\n\n  * `:prefer_fenced` (`t:boolean/0`) - Prefer fenced code blocks when outputting CommonMark. The default value is `false`.\n\n  * `:figure_with_caption` (`t:boolean/0`) - Render the image as a figure element with the title as its caption. The default value is `false`.\n\n  * `:tasklist_classes` (`t:boolean/0`) - Add classes to the output of the tasklist extension. This allows tasklists to be styled. The default value is `false`.\n\n  * `:ol_width` (`t:integer/0`) - Render ordered list with a minimum marker width. Having a width lower than 3 doesn't do anything. The default value is `1`.\n\n  * `:experimental_minimize_commonmark` (`t:boolean/0`) - Minimise escapes used in CommonMark output (`-t commonmark`) by removing each individually and seeing if the resulting document roundtrips.\n    Brute-force and expensive, but produces nicer output.\n    Note that the result may not in fact be minimal. The default value is `false`.\n\n* `:syntax_highlight` - Apply syntax highlighting to code blocks.\n\n    Examples:\n\n        syntax_highlight: [formatter: {:html_inline, theme: \"github_dark\"}]\n\n        syntax_highlight: [formatter: {:html_linked, theme: \"github_light\"}]\n\n    See [Autumn](https://hexdocs.pm/autumn) for more info and examples.\n\n  The default value is `[formatter: {:html_inline, [theme: \"onedark\"]}]`.\n\n* `:sanitize` - Cleans HTML using [ammonia](https://crates.io/crates/ammonia) after rendering.\n\n  It's disabled by default but you can enable its [conservative set of default options](https://docs.rs/ammonia/latest/ammonia/fn.clean.html) as:\n\n      [sanitize: MDEx.default_sanitize_options()]\n\n  Or customize one of the options. For example, to disallow ` ` tags:\n\n      [sanitize: [rm_tags: [\"a\"]]]\n\n  In the example above it will append `rm_tags: [\"a\"]` into the default set of options, essentially the same as:\n\n      sanitize = Keyword.put(MDEx.default_sanitize_options(), :rm_tags, [\"a\"])\n      [sanitize: sanitize]\n\n  See the [Safety](#module-safety) section for more info.\n\n  The default value is `nil`.\n\n* `:features` (`t:keyword/0`) - *This option is deprecated. Use :syntax_highlight or :sanitize instead.*\n\n  * `:sanitize` - *This option is deprecated. Use :sanitize (in :options) instead.*\n\n  * `:syntax_highlight_theme` - *This option is deprecated. Use :syntax_highlight (in :options) instead.*\n\n  * `:syntax_highlight_inline_style` (`t:boolean/0`) - *This option is deprecated. Use :syntax_highlight (in :options) instead.*","ref":"MDEx.html#t:options/0-options"},{"type":"function","title":"MDEx.parse_document/2","doc":"Parse `source` and returns `MDEx.Document`.\n\nSource can be either a Markdown string or a tagged JSON string.","ref":"MDEx.html#parse_document/2"},{"type":"function","title":"Examples - MDEx.parse_document/2","doc":"* Parse Markdown with default options:\n\n      iex> MDEx.parse_document!(\"\"\"\n      ...> # Languages\n      ...>\n      ...> - Elixir\n      ...> - Rust\n      ...> \"\"\")\n      %MDEx.Document{\n        nodes: [\n          %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Languages\"}], level: 1, setext: false},\n          %MDEx.List{\n            nodes: [\n              %MDEx.ListItem{\n                nodes: [%MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"Elixir\"}]}],\n                list_type: :bullet,\n                marker_offset: 0,\n                padding: 2,\n                start: 1,\n                delimiter: :period,\n                bullet_char: \"-\",\n                tight: false\n              },\n              %MDEx.ListItem{\n                nodes: [%MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"Rust\"}]}],\n                list_type: :bullet,\n                marker_offset: 0,\n                padding: 2,\n                start: 1,\n                delimiter: :period,\n                bullet_char: \"-\",\n                tight: false\n              }\n            ],\n            list_type: :bullet,\n            marker_offset: 0,\n            padding: 2,\n            start: 1,\n            delimiter: :period,\n            bullet_char: \"-\",\n            tight: true\n          }\n        ]\n      }\n\n* Parse Markdown with custom options:\n\n      iex> MDEx.parse_document!(\"Darth Vader is ||Luke's father||\", extension: [spoiler: true])\n      %MDEx.Document{\n        nodes: [\n          %MDEx.Paragraph{\n            nodes: [\n              %MDEx.Text{literal: \"Darth Vader is \"},\n              %MDEx.SpoileredText{nodes: [%MDEx.Text{literal: \"Luke's father\"}]}\n            ]\n          }\n        ]\n      }\n\n* Parse JSON:\n\n      iex> json = ~s|{\"nodes\":[{\"nodes\":[{\"literal\":\"Title\",\"node_type\":\"MDEx.Text\"}],\"level\":1,\"setext\":false,\"node_type\":\"MDEx.Heading\"}],\"node_type\":\"MDEx.Document\"}|\n      iex> MDEx.parse_document!({:json, json})\n      %MDEx.Document{\n        nodes: [\n          %MDEx.Heading{\n            nodes: [%MDEx.Text{literal: \"Title\"} ],\n            level: 1,\n            setext: false\n          }\n        ]\n      }","ref":"MDEx.html#parse_document/2-examples"},{"type":"function","title":"MDEx.parse_document!/2","doc":"Same as `parse_document/2` but raises if the parsing fails.","ref":"MDEx.html#parse_document!/2"},{"type":"function","title":"MDEx.parse_fragment/2","doc":"Parse a `markdown` string and returns only the node that represents the fragment.\n\nUsually that means filtering out the parent document and paragraphs.\n\nThat's useful to generate fragment nodes and inject them into the document\nwhen you're manipulating it.\n\nUse `parse_document/2` to generate a complete document.\n\n> #### Experimental {: .warning}\n>\n> Consider this function experimental and subject to change.","ref":"MDEx.html#parse_fragment/2"},{"type":"function","title":"Examples - MDEx.parse_fragment/2","doc":"iex> MDEx.parse_fragment(\"# Elixir\")\n    {:ok, %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Elixir\"}], level: 1, setext: false}}\n\n    iex> MDEx.parse_fragment(\" Elixir \")\n    {:ok, %MDEx.HtmlBlock{nodes: [], block_type: 6, literal: \" Elixir \\n\"}}","ref":"MDEx.html#parse_fragment/2-examples"},{"type":"function","title":"MDEx.parse_fragment!/2","doc":"Same as `parse_fragment/2` but raises if the parsing fails or returns `nil`.\n\n> #### Experimental {: .warning}\n>\n> Consider this function experimental and subject to change.","ref":"MDEx.html#parse_fragment!/2"},{"type":"type","title":"MDEx.parse_options/0","doc":"List of [comrak parse options](https://docs.rs/comrak/latest/comrak/struct.ParseOptions.html).","ref":"MDEx.html#t:parse_options/0"},{"type":"type","title":"MDEx.render_options/0","doc":"List of [comrak render options](https://docs.rs/comrak/latest/comrak/struct.RenderOptions.html).","ref":"MDEx.html#t:render_options/0"},{"type":"function","title":"MDEx.safe_html/2","doc":"Utility function to sanitize and escape HTML.","ref":"MDEx.html#safe_html/2"},{"type":"function","title":"Examples - MDEx.safe_html/2","doc":"iex> MDEx.safe_html(\" console.log('attack') \")\n    \"\"\n\n    iex> MDEx.safe_html(\" Hello \")\n    \"Hello\"\n\n    iex> MDEx.safe_html(\" Hello \", sanitize: [add_tags: [\"custom_tag\"]], escape: [content: false])\n    \" Hello \"\n\n    iex> MDEx.safe_html(\" {'Example:'}  {:ok, 'MDEx'} \")\n    \"&lt;h1&gt;{&#x27;Example:&#x27;}&lt;&#x2f;h1&gt;&lt;code&gt;&lbrace;:ok, &#x27;MDEx&#x27;&rbrace;&lt;&#x2f;code&gt;\"\n\n    iex> MDEx.safe_html(\" {'Example:'}  {:ok, 'MDEx'} \", escape: [content: false])\n    \" {'Example:'}  &lbrace;:ok, 'MDEx'&rbrace; \"","ref":"MDEx.html#safe_html/2-examples"},{"type":"function","title":"Options - MDEx.safe_html/2","doc":"- `:sanitize` - cleans HTML after rendering. Defaults to `MDEx.default_sanitize_options/0`.\n      - `keyword` - `t:sanitize_options/0`\n      - `nil` - do not sanitize output.\n\n  - `:escape` - which entities should be escaped. Defaults to `[:content, :curly_braces_in_code]`.\n      - `:content` - escape common chars like `<`, `>`, `&`, and others in the HTML content;\n      - `:curly_braces_in_code` - escape `{` and `}` only inside ` ` tags, particularly useful for compiling HTML in LiveView;","ref":"MDEx.html#safe_html/2-options"},{"type":"type","title":"MDEx.sanitize_options/0","doc":"List of [ammonia options](https://docs.rs/ammonia/latest/ammonia/struct.Builder.html).","ref":"MDEx.html#t:sanitize_options/0"},{"type":"type","title":"MDEx.source/0","doc":"Input source document.","ref":"MDEx.html#t:source/0"},{"type":"type","title":"Examples - MDEx.source/0","doc":"* From Markdown to HTML\n\n      iex> MDEx.to_html!(\"# Hello\")\n      \" Hello \"\n\n* From Markdown to `MDEx.Document`\n\n      iex> MDEx.parse_document!(\"Hello\")\n      %MDEx.Document{\n        nodes: [\n          %MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"Hello\"}]}\n        ]\n      }\n\n* From `MDEx.Document` to HTML\n\n      iex> MDEx.to_html!(%MDEx.Document{\n      ...>   nodes: [\n      ...>     %MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"Hello\"}]}\n      ...>   ]\n      ...> })\n      \" Hello \"\n\nYou can also leverage `MDEx.Document` as an intermediate data type to convert between formats:\n\n* From JSON to HTML:\n\n      iex> json = ~s|{\"nodes\":[{\"nodes\":[{\"literal\":\"Hello\",\"node_type\":\"MDEx.Text\"}],\"level\":1,\"setext\":false,\"node_type\":\"MDEx.Heading\"}],\"node_type\":\"MDEx.Document\"}|\n      iex> {:json, json} |> MDEx.parse_document!() |> MDEx.to_html!()\n      \" Hello \"","ref":"MDEx.html#t:source/0-examples"},{"type":"type","title":"MDEx.syntax_highlight_options/0","doc":"Syntax Highlight code blocks using [autumn](https://hexdocs.pm/autumn).","ref":"MDEx.html#t:syntax_highlight_options/0"},{"type":"function","title":"MDEx.to_commonmark/1","doc":"","ref":"MDEx.html#to_commonmark/1"},{"type":"function","title":"MDEx.to_commonmark/2","doc":"","ref":"MDEx.html#to_commonmark/2"},{"type":"function","title":"MDEx.to_commonmark!/1","doc":"","ref":"MDEx.html#to_commonmark!/1"},{"type":"function","title":"MDEx.to_commonmark!/2","doc":"","ref":"MDEx.html#to_commonmark!/2"},{"type":"function","title":"MDEx.to_html/2","doc":"Convert Markdown, `MDEx.Document`, or `MDEx.Pipe` to HTML.","ref":"MDEx.html#to_html/2"},{"type":"function","title":"Examples - MDEx.to_html/2","doc":"iex> MDEx.to_html(\"# MDEx\")\n    {:ok, \" MDEx \"}\n\n    iex> MDEx.to_html(\"Implemented with:\\n1. Elixir\\n2. Rust\")\n    {:ok, \" Implemented with: \\n \\n Elixir \\n Rust \\n \"}\n\n    iex> MDEx.to_html(%MDEx.Document{nodes: [%MDEx.Heading{nodes: [%MDEx.Text{literal: \"MDEx\"}], level: 3, setext: false}]})\n    {:ok, \" MDEx \"}\n\n    iex> MDEx.to_html(\"Hello ~world~ there\", extension: [strikethrough: true])\n    {:ok, \" Hello  world  there \"}\n\n    iex> MDEx.to_html(\" visit https://beaconcms.org \", extension: [autolink: true], render: [unsafe: true])\n    {:ok, \"  visit  https://beaconcms.org   \"}\n\n Fragments of a document are also supported:\n\n    iex> MDEx.to_html(%MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"MDEx\"}]})\n    {:ok, \" MDEx \"}","ref":"MDEx.html#to_html/2-examples"},{"type":"function","title":"MDEx.to_html!/2","doc":"Same as `to_html/2` but raises error if the conversion fails.","ref":"MDEx.html#to_html!/2"},{"type":"function","title":"MDEx.to_json/1","doc":"Convert Markdown, `MDEx.Document`, or `MDEx.Pipe` to JSON using default options.\n\nUse `to_json/2` to pass custom options.","ref":"MDEx.html#to_json/1"},{"type":"function","title":"Examples - MDEx.to_json/1","doc":"iex> MDEx.to_json(\"# Hello\")\n    {:ok, ~s|{\"nodes\":[{\"nodes\":[{\"literal\":\"Hello\",\"node_type\":\"MDEx.Text\"}],\"level\":1,\"setext\":false,\"node_type\":\"MDEx.Heading\"}],\"node_type\":\"MDEx.Document\"}|}\n\n    iex> MDEx.to_json(\"1. First\\n2. Second\")\n    {:ok, ~s|{\"nodes\":[{\"start\":1,\"nodes\":[{\"start\":1,\"nodes\":[{\"nodes\":[{\"literal\":\"First\",\"node_type\":\"MDEx.Text\"}],\"node_type\":\"MDEx.Paragraph\"}],\"delimiter\":\"period\",\"padding\":3,\"list_type\":\"ordered\",\"marker_offset\":0,\"bullet_char\":\"\",\"tight\":false,\"is_task_list\":false,\"node_type\":\"MDEx.ListItem\"},{\"start\":2,\"nodes\":[{\"nodes\":[{\"literal\":\"Second\",\"node_type\":\"MDEx.Text\"}],\"node_type\":\"MDEx.Paragraph\"}],\"delimiter\":\"period\",\"padding\":3,\"list_type\":\"ordered\",\"marker_offset\":0,\"bullet_char\":\"\",\"tight\":false,\"is_task_list\":false,\"node_type\":\"MDEx.ListItem\"}],\"delimiter\":\"period\",\"padding\":3,\"list_type\":\"ordered\",\"marker_offset\":0,\"bullet_char\":\"\",\"tight\":true,\"is_task_list\":false,\"node_type\":\"MDEx.List\"}],\"node_type\":\"MDEx.Document\"}|}\n\n    iex> MDEx.to_json(%MDEx.Document{nodes: [%MDEx.Heading{nodes: [%MDEx.Text{literal: \"Hello\"}], level: 3, setext: false}]})\n    {:ok, ~s|{\"nodes\":[{\"nodes\":[{\"literal\":\"Hello\",\"node_type\":\"MDEx.Text\"}],\"level\":3,\"setext\":false,\"node_type\":\"MDEx.Heading\"}],\"node_type\":\"MDEx.Document\"}|}\n\nFragments of a document are also supported:\n\n    iex> MDEx.to_json(%MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"Hello\"}]})\n    {:ok, ~s|{\"nodes\":[{\"nodes\":[{\"literal\":\"Hello\",\"node_type\":\"MDEx.Text\"}],\"node_type\":\"MDEx.Paragraph\"}],\"node_type\":\"MDEx.Document\"}|}","ref":"MDEx.html#to_json/1-examples"},{"type":"function","title":"MDEx.to_json/2","doc":"Convert Markdown, `MDEx.Document`, or `MDEx.Pipe` to JSON using custom options.","ref":"MDEx.html#to_json/2"},{"type":"function","title":"Examples - MDEx.to_json/2","doc":"iex> MDEx.to_json(\"Hello ~world~\", extension: [strikethrough: true])\n    {:ok, ~s|{\"nodes\":[{\"nodes\":[{\"literal\":\"Hello \",\"node_type\":\"MDEx.Text\"},{\"nodes\":[{\"literal\":\"world\",\"node_type\":\"MDEx.Text\"}],\"node_type\":\"MDEx.Strikethrough\"}],\"node_type\":\"MDEx.Paragraph\"}],\"node_type\":\"MDEx.Document\"}|}","ref":"MDEx.html#to_json/2-examples"},{"type":"function","title":"MDEx.to_json!/1","doc":"Same as `to_json/1` but raises an error if the conversion fails.","ref":"MDEx.html#to_json!/1"},{"type":"function","title":"MDEx.to_json!/2","doc":"Same as `to_json/2` but raises error if the conversion fails.","ref":"MDEx.html#to_json!/2"},{"type":"function","title":"MDEx.to_markdown/1","doc":"Convert `MDEx.Document` or `MDEx.Pipe` to Markdown using default options.\n\nUse `to_markdown/2` to pass custom options.","ref":"MDEx.html#to_markdown/1"},{"type":"function","title":"Example - MDEx.to_markdown/1","doc":"iex> MDEx.to_markdown(%MDEx.Document{nodes: [%MDEx.Heading{nodes: [%MDEx.Text{literal: \"Hello\"}], level: 3, setext: false}]})\n    {:ok, \"","ref":"MDEx.html#to_markdown/1-example"},{"type":"function","title":"Hello\"} - MDEx.to_markdown/1","doc":"","ref":"MDEx.html#to_markdown/1-hello"},{"type":"function","title":"MDEx.to_markdown/2","doc":"Convert `MDEx.Document` or `MDEx.Pipe` to Markdown using custom options.","ref":"MDEx.html#to_markdown/2"},{"type":"function","title":"MDEx.to_markdown!/1","doc":"Same as `to_markdown/1` but raises `MDEx.DecodeError` if the conversion fails.","ref":"MDEx.html#to_markdown!/1"},{"type":"function","title":"MDEx.to_markdown!/2","doc":"Same as `to_markdown/2` but raises `MDEx.DecodeError` if the conversion fails.","ref":"MDEx.html#to_markdown!/2"},{"type":"function","title":"MDEx.to_xml/2","doc":"Convert Markdown, `MDEx.Document`, or `MDEx.Pipe` to XML.","ref":"MDEx.html#to_xml/2"},{"type":"function","title":"Examples - MDEx.to_xml/2","doc":"iex> {:ok, xml} = MDEx.to_xml(\"Hello ~world~ there\", extension: [strikethrough: true])\n    iex> xml\n    ~s|<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n    <!DOCTYPE document SYSTEM \"CommonMark.dtd\">\n     \n       \n         Hello  \n         \n           world \n         \n          there \n       \n     |\n\n    iex> {:ok, xml} = MDEx.to_xml(\" visit https://beaconcms.org \", extension: [autolink: true], render: [unsafe: true])\n    iex> xml\n    ~s|<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n    <!DOCTYPE document SYSTEM \"CommonMark.dtd\">\n     \n       \n         &lt;marquee&gt; \n         visit  \n         \n           https://beaconcms.org \n         \n         &lt;/marquee&gt; \n       \n     |\n\nFragments of a document are also supported:\n\n    iex> {:ok, xml} = MDEx.to_xml(%MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"MDEx\"}]})\n    iex> xml\n    ~s|<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n    <!DOCTYPE document SYSTEM \"CommonMark.dtd\">\n     \n       \n         MDEx \n       \n     |","ref":"MDEx.html#to_xml/2-examples"},{"type":"function","title":"MDEx.to_xml!/2","doc":"Same as `to_xml/2` but raises error if the conversion fails.","ref":"MDEx.html#to_xml!/2"},{"type":"function","title":"MDEx.traverse_and_update/2","doc":"Low-level function to traverse and update the Markdown document preserving the tree structure format.\n\nSee `MDEx.Document` for more information about the tree structure and for higher-level functions\nusing the Access and Enumerable protocols.","ref":"MDEx.html#traverse_and_update/2"},{"type":"function","title":"Examples - MDEx.traverse_and_update/2","doc":"Traverse an entire Markdown document:\n\n    iex> import MDEx.Sigil\n    iex> doc = ~MD\"\"\"\n    ...> # Languages\n    ...>\n    ...> `elixir`\n    ...>\n    ...> `rust`\n    ...> \"\"\"\n    iex> MDEx.traverse_and_update(doc, fn\n    ...>   %MDEx.Code{literal: \"elixir\"} = node -> %{node | literal: \"ex\"}\n    ...>   %MDEx.Code{literal: \"rust\"} = node -> %{node | literal: \"rs\"}\n    ...>   node -> node\n    ...> end)\n    %MDEx.Document{\n      nodes: [\n        %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Languages\"}], level: 1, setext: false},\n        %MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"ex\"}]},\n        %MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"rs\"}]}\n      ]\n    }\n\n  Or fragments of a document:\n\n    iex> fragment = MDEx.parse_fragment!(\"Lang: `elixir`\")\n    iex> MDEx.traverse_and_update(fragment, fn\n    ...>   %MDEx.Code{literal: \"elixir\"} = node -> %{node | literal: \"ex\"}\n    ...>   node -> node\n    ...> end)\n    %MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"Lang: \"}, %MDEx.Code{num_backticks: 1, literal: \"ex\"}]}","ref":"MDEx.html#traverse_and_update/2-examples"},{"type":"function","title":"MDEx.traverse_and_update/3","doc":"Low-level function to traverse and update the Markdown document preserving the tree structure format and keeping an accumulator.\n\nSee `MDEx.Document` for more information about the tree structure and for higher-level functions\nusing the Access and Enumerable protocols.","ref":"MDEx.html#traverse_and_update/3"},{"type":"function","title":"Example - MDEx.traverse_and_update/3","doc":"iex> import MDEx.Sigil\n    iex> doc = ~MD\"\"\"\n    ...> # Languages\n    ...>\n    ...> `elixir`\n    ...>\n    ...> `rust`\n    ...> \"\"\"\n    iex> MDEx.traverse_and_update(doc, 0, fn\n    ...>   %MDEx.Code{literal: \"elixir\"} = node, acc -> {%{node | literal: \"ex\"}, acc + 1}\n    ...>   %MDEx.Code{literal: \"rust\"} = node, acc -> {%{node | literal: \"rs\"}, acc + 1}\n    ...>   node, acc -> {node, acc}\n    ...> end)\n    {%MDEx.Document{\n      nodes: [\n        %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Languages\"}], level: 1, setext: false},\n        %MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"ex\"}]},\n        %MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"rs\"}]}\n      ]\n    }, 2}\n\nAlso works with fragments.","ref":"MDEx.html#traverse_and_update/3-example"},{"type":"module","title":"MDEx.Document","doc":"Tree representation of a Markdown document.\n\n```elixir\n%MDEx.Document{\n  nodes: [\n    %MDEx.Paragraph{\n      nodes: [\n        %MDEx.Code{num_backticks: 1, literal: \"Elixir\"}\n      ]\n    }\n  ]\n}\n```\n\nEach node may contain attributes and children nodes as in the example above where `MDEx.Document`\ncontains a `MDEx.Paragraph` node which contains a `MDEx.Code` node with the attributes `:num_backticks` and `:literal`.\n\nYou can check out each node's documentation in the `Document Nodes` section, for example `MDEx.HtmlBlock`.\n\nThe wrapping `MDEx.Document` module represents the root of a document and it implements some behaviours and protocols\nto enable operations to fetch, update, and manipulate the document tree.\n\nLet's go through these operations in the examples below.\n\nIn these examples we will be using the [~MD](https://hexdocs.pm/mdex/MDEx.Sigil.html#sigil_MD/2) sigil.","ref":"MDEx.Document.html"},{"type":"module","title":"String.Chars - MDEx.Document","doc":"Calling `Kernel.to_string/1` will format it as CommonMark text:\n\n```elixir\niex> to_string(~MD[# Hello])\n\"# Hello\"\n```\n\nFragments (nodes without the parent `%Document{}`) are also formatted:\n\n```elixir\niex> to_string(%MDEx.Heading{nodes: [%MDEx.Text{literal: \"Hello\"}], level: 1})\n\"# Hello\"\n```","ref":"MDEx.Document.html#module-string-chars"},{"type":"module","title":"Access - MDEx.Document","doc":"The `Access` behaviour gives you the ability to fetch and update nodes using different types of keys.\n\nStarting with a simple Markdown document with a single heading and a text,\nlet's fetch only the text node by matching the `MDEx.Text` node:\n\n```elixir\niex> ~MD[# Hello][%MDEx.Text{literal: \"Hello\"}]\n[%MDEx.Text{literal: \"Hello\"}]\n```\n\nThat's essentially the same as:\n\n```elixir\ndoc = %MDEx.Document{nodes: [%MDEx.Heading{nodes: [%MDEx.Text{literal: \"Hello\"}], level: 1, setext: false}]},\n\nEnum.filter(\n  doc,\n  fn node -> node == %MDEx.Text{literal: \"Hello\"} end\n)\n```\n\nThe key can also be modules, atoms, and even functions! For example:\n\n- Fetches all Code nodes, either by `MDEx.Code` module or the `:code` atom representing the Code node\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Languages\n...>\n...> `elixir`\n...>\n...> `rust`\n...> \"\"\"\niex> doc[MDEx.Code]\n[%MDEx.Code{num_backticks: 1, literal: \"elixir\"}, %MDEx.Code{num_backticks: 1, literal: \"rust\"}]\niex> doc[:code]\n[%MDEx.Code{num_backticks: 1, literal: \"elixir\"}, %MDEx.Code{num_backticks: 1, literal: \"rust\"}]\n```\n\n- Dynamically fetch Code nodes where the `:literal` (node content) starts with `\"eli\"` using a function to filter the result\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Languages\n...>\n...> `elixir`\n...>\n...> `rust`\n...> \"\"\"\niex> doc[fn node -> String.starts_with?(Map.get(node, :literal, \"\"), \"eli\") end]\n[%MDEx.Code{num_backticks: 1, literal: \"elixir\"}]\n```\n\nThat's the most flexible option, in the case where struct, modules, or atoms are not enough to match the node you want.\n\nThis protocol also allows us to update nodes that matches a selector.\nIn the example below we'll capitalize the content of all `MDEx.Code` nodes:\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Languages\n...>\n...> `elixir`\n...>\n...> `rust`\n...>\n...> Continue...\n...> \"\"\"\niex> update_in(doc, [:document, Access.key!(:nodes), Access.all(), :code, Access.key!(:literal)], fn literal ->\n...>   String.upcase(literal)\n...> end)\n%MDEx.Document{\n  nodes: [\n    %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Languages\"}], level: 1, setext: false},\n    %MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"ELIXIR\"}]},\n    %MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"RUST\"}]},\n    %MDEx.Paragraph{nodes: [%MDEx.Text{literal: \"Continue...\"}]}\n  ]\n}\n```","ref":"MDEx.Document.html#module-access"},{"type":"module","title":"Enumerable - MDEx.Document","doc":"Probably the most used protocol in Elixir, it allows us to call `Enum` functions to manipulate the document. Let's see some examples:\n\n* Count the nodes in a document:\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Languages\n...>\n...> `elixir`\n...>\n...> `rust`\n...> \"\"\"\niex> Enum.count(doc)\n7\n```\n\n* Count how many nodes have the `:literal` attribute:\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Languages\n...>\n...> `elixir`\n...>\n...> `rust`\n...> \"\"\"\niex> Enum.reduce(doc, 0, fn\n...>   %{literal: _literal}, acc -> acc + 1\n...>\n...>   _node, acc -> acc\n...> end)\n3\n```\n\n* Returns true if node is member of the document:\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Languages\n...>\n...> `elixir`\n...>\n...> `rust`\n...> \"\"\"\niex> Enum.member?(doc, %MDEx.Code{literal: \"elixir\", num_backticks: 1})\ntrue\n```\n\n* Map each node:\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Languages\n...>\n...> `elixir`\n...>\n...> `rust`\n...> \"\"\"\niex> Enum.map(doc, fn %node{} -> inspect(node) end)\n[\"MDEx.Document\", \"MDEx.Heading\", \"MDEx.Text\", \"MDEx.Paragraph\", \"MDEx.Code\", \"MDEx.Paragraph\", \"MDEx.Code\"]\n```","ref":"MDEx.Document.html#module-enumerable"},{"type":"module","title":"Traverse and Update - MDEx.Document","doc":"You can also use the low-level `MDEx.traverse_and_update/2` and `MDEx.traverse_and_update/3` APIs\nto traverse each node of the AST and either update the nodes or do some calculation with an accumulator.","ref":"MDEx.Document.html#module-traverse-and-update"},{"type":"module","title":"Examples - MDEx.Document","doc":"#### Update all code block nodes filtees by the `selector` function\n\n_Add line \"// Modified\" in Rust block codes_:\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Code Examples\n...>\n...> ```elixir\n...> def hello do\n...>   :world\n...> end\n...> ```\n...>\n...> ```rust\n...> fn main() {\n...>   println!(\"Hello\");\n...> }\n...> ```\n...> \"\"\"\niex> selector = fn\n...>   %MDEx.CodeBlock{info: \"rust\"} -> true\n...>   _ -> false\n...> end\niex> update_in(doc, [:document, Access.key!(:nodes), Access.all(), selector], fn node ->\n...>   %{node | literal: \"// Modified\\n\" <> node.literal}\n...> end)\n%MDEx.Document{\n  nodes: [\n    %MDEx.Heading{\n      nodes: [%MDEx.Text{literal: \"Code Examples\"}],\n      level: 1,\n      setext: false\n    },\n    %MDEx.CodeBlock{\n      info: \"elixir\",\n      literal: \"def hello do\\n  :world\\nend\\n\"\n    },\n    %MDEx.CodeBlock{\n      info: \"rust\",\n      literal: \"// Modified\\nfn main() {\\n  println!(\\\"Hello\\\");\\n}\\n\"\n    }\n  ]\n}\n```\n\n#### Collect headings by level\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Main Title\n...>\n...>","ref":"MDEx.Document.html#module-examples"},{"type":"module","title":"Section 1 - MDEx.Document","doc":"...>\n...>","ref":"MDEx.Document.html#module-section-1"},{"type":"module","title":"Subsection - MDEx.Document","doc":"...>\n...>","ref":"MDEx.Document.html#module-subsection"},{"type":"module","title":"Section 2 - MDEx.Document","doc":"...> \"\"\"\niex> Enum.reduce(doc, %{}, fn\n...>   %MDEx.Heading{level: level, nodes: [%MDEx.Text{literal: text}]}, acc ->\n...>     Map.update(acc, level, [text], &[text | &1])\n...>   _node, acc -> acc\n...> end)\n%{\n  1 => [\"Main Title\"],\n  2 => [\"Section 2\", \"Section 1\"],\n  3 => [\"Subsection\"]\n}\n```\n\n#### Extract and transform task list items\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Todo List\n...>\n...> - [ ] Buy groceries\n...> - [x] Call mom\n...> - [ ] Read book\n...> \"\"\"\niex> Enum.map(doc, fn\n...>   %MDEx.TaskItem{checked: checked, nodes: [%MDEx.Paragraph{nodes: [%MDEx.Text{literal: text}]}]} ->\n...>     {checked, text}\n...>   _ -> nil\n...> end)\n...> |> Enum.reject(&is_nil/1)\n[\n  {false, \"Buy groceries\"},\n  {true, \"Call mom\"},\n  {false, \"Read book\"}\n]\n```\n\n#### Bump all heading levels, except level 6\n\n```elixir\niex> doc = ~MD\"\"\"\n...> # Main Title\n...>\n...>","ref":"MDEx.Document.html#module-section-2"},{"type":"module","title":"Subtitle - MDEx.Document","doc":"...>\n...> ###### Notes\n...> \"\"\"\niex> selector = fn\n...>   %MDEx.Heading{level: level} when level < 6 -> true\n...>   _ -> false\n...> end\niex> update_in(doc, [:document, Access.key!(:nodes), Access.all(), selector], fn node ->\n...>   %{node | level: node.level + 1}\n...> end)\n%MDEx.Document{\n  nodes: [\n    %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Main Title\"}], level: 2, setext: false},\n    %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Subtitle\"}], level: 3, setext: false},\n    %MDEx.Heading{nodes: [%MDEx.Text{literal: \"Notes\"}], level: 6, setext: false}\n  ]\n}\n```","ref":"MDEx.Document.html#module-subtitle"},{"type":"function","title":"MDEx.Document.fetch/2","doc":"Callback implementation for `Access.fetch/2`.\n\nSee the [Access](#module-access) section for examples.","ref":"MDEx.Document.html#fetch/2"},{"type":"function","title":"MDEx.Document.get_and_update/3","doc":"Callback implementation for `Access.get_and_update/3`.\n\nSee the [Access](#module-access) section for examples.","ref":"MDEx.Document.html#get_and_update/3"},{"type":"type","title":"MDEx.Document.md_node/0","doc":"Fragment of a Markdown document, a single node. May contain children nodes.","ref":"MDEx.Document.html#t:md_node/0"},{"type":"function","title":"MDEx.Document.pop/3","doc":"Callback implementation for `Access.fetch/2`.\n\nSee the [Access](#module-access) section for examples.","ref":"MDEx.Document.html#pop/3"},{"type":"type","title":"MDEx.Document.selector/0","doc":"Selector used to match nodes in the document.\n\nValid selectors can be the module or struct, an atom representing the node name, or a function that receives a node and returns a boolean.\n\nSee `MDEx.Document` for more info and examples.","ref":"MDEx.Document.html#t:selector/0"},{"type":"type","title":"MDEx.Document.t/0","doc":"Tree root of a Markdown document, including all children nodes.","ref":"MDEx.Document.html#t:t/0"},{"type":"function","title":"MDEx.Document.wrap/1","doc":"","ref":"MDEx.Document.html#wrap/1"},{"type":"module","title":"MDEx.Pipe","doc":"MDEx.Pipe is a Req-like API to transform Markdown documents through a series of steps in a pipeline.\n\nIts main use case it to enable plugins, for example:\n\n    document = \"\"\"\n    # Project Diagram\n\n    ```mermaid\n    graph TD\n        A[Enter Chart Definition] --> B(Preview)\n        B --> C{decide}\n        C --> D[Keep]\n        C --> E[Edit Definition]\n        E --> B\n        D --> F[Save Image and Code]\n        F --> B\n    ```\n    \"\"\"\n\n    MDEx.new()\n    |> MDExMermaid.attach(mermaid_version: \"11\")\n    |> MDEx.to_html(document: document)\n\nTo understand how it works, let's write that Mermaid plugin.","ref":"MDEx.Pipe.html"},{"type":"module","title":"Writing Plugins - MDEx.Pipe","doc":"Let's start with a simple plugin as example to render Mermaid diagrams.\n\nIn order to render Mermaid diagrams, we need to inject a ` ` into the document,\nas outlined in their [docs](https://mermaid.js.org/intro/#installation):\n\n     \n      import mermaid from 'https://cdn.jsdelivr.net/npm/mermaid@11/dist/mermaid.esm.min.mjs';\n      mermaid.initialize({ startOnLoad: true });\n     \n\nNote that the package version is specified in the URL, so we'll add an option\n`:mermaid_version` to the plugin to let users specify the version they want to use.\n\nBy default, we'll use the latest version:\n\n    MDEx.new() |> MDExMermaid.attach()\n\nBut users can override it:\n\n    MDEx.new() |> MDExMermaid.attach(mermaid_version: \"11\")\n\nLet's get into the actual code, with comments to explain each part:\n\n    defmodule MDExMermaid do\n      alias MDEx.Pipe\n\n      @latest_version \"11\"\n\n      def attach(pipe, options \\ []) do\n        pipe\n        # register option with prefix `:mermaid_` to avoid conflicts with other plugins\n        |> Pipe.register_options([:mermaid_version])\n        #  merge all options given by users\n        |> Pipe.put_options(options)\n        # actual steps to manipulate the document\n        # see respective Pipe functions for more info\n        |> Pipe.append_steps(enable_unsafe: &enable_unsafe/1)\n        |> Pipe.append_steps(inject_script: &inject_script/1)\n        |> Pipe.append_steps(update_code_blocks: &update_code_blocks/1)\n      end\n\n      # to render raw html and   tags\n      defp enable_unsafe(pipe) do\n        Pipe.put_render_options(pipe, unsafe: true)\n      end\n\n      defp inject_script(pipe) do\n        version = Pipe.get_option(pipe, :mermaid_version, @latest_version)\n\n        script_node =\n          %MDEx.HtmlBlock{\n            literal: \"\"\"\n             \n              import mermaid from 'https://cdn.jsdelivr.net/npm/mermaid@#{version}/dist/mermaid.esm.min.mjs';\n              mermaid.initialize({ startOnLoad: true });\n             \n            \"\"\"\n          }\n\n        Pipe.put_node_in_document_root(pipe, script_node)\n      end\n\n      defp update_code_blocks(pipe) do\n        selector = fn\n          %MDEx.CodeBlock{info: \"mermaid\"} -> true\n          _ -> false\n        end\n\n        Pipe.update_nodes(\n          pipe,\n          selector,\n          &%MDEx.HtmlBlock{literal: \" #{&1.literal} \", nodes: &1.nodes}\n        )\n      end\n    end\n\nNow we can `attach/1` that plugin into any MDEx pipeline to render Mermaid diagrams.","ref":"MDEx.Pipe.html#module-writing-plugins"},{"type":"function","title":"MDEx.Pipe.append_steps/2","doc":"Appends steps to the end of the existing pipeline's step list.","ref":"MDEx.Pipe.html#append_steps/2"},{"type":"function","title":"Examples - MDEx.Pipe.append_steps/2","doc":"* Update an `:extension` option:\n\n      iex> pipe = MDEx.new()\n      iex> pipe = MDEx.Pipe.append_steps(\n      ...>   pipe,\n      ...>   enable_tables: fn pipe -> MDEx.Pipe.put_extension_options(pipe, table: true) end\n      ...> )\n      iex> pipe |> MDEx.Pipe.run() |> MDEx.Pipe.get_option(:extension)\n      [table: true]","ref":"MDEx.Pipe.html#append_steps/2-examples"},{"type":"function","title":"MDEx.Pipe.get_option/3","doc":"Retrieves an option value from the pipeline.","ref":"MDEx.Pipe.html#get_option/3"},{"type":"function","title":"MDEx.Pipe.get_private/3","doc":"Retrieves a private value from the pipeline.","ref":"MDEx.Pipe.html#get_private/3"},{"type":"function","title":"MDEx.Pipe.get_sanitize_option/3","doc":"Retrieves one of the `t:MDEx.sanitize_options/0` options from the pipeline.","ref":"MDEx.Pipe.html#get_sanitize_option/3"},{"type":"function","title":"MDEx.Pipe.halt/1","doc":"Halts the pipeline execution.\n\nThis function is used to stop the pipeline from processing any further steps. Once a pipeline\nis halted, no more steps will be executed. This is useful for plugins that need to stop\nprocessing when certain conditions are met or when an error occurs.","ref":"MDEx.Pipe.html#halt/1"},{"type":"function","title":"Examples - MDEx.Pipe.halt/1","doc":"iex> pipe = MDEx.Pipe.halt(MDEx.new())\n    iex> pipe.halted\n    true","ref":"MDEx.Pipe.html#halt/1-examples"},{"type":"function","title":"MDEx.Pipe.halt/2","doc":"Halts the pipeline execution with an exception.","ref":"MDEx.Pipe.html#halt/2"},{"type":"function","title":"MDEx.Pipe.is_sanitize_enabled/1","doc":"Returns `true` if the pipeline has the `:sanitize` option set, otherwise `false`.","ref":"MDEx.Pipe.html#is_sanitize_enabled/1"},{"type":"function","title":"MDEx.Pipe.prepend_steps/2","doc":"Prepends steps to the beginning of the existing pipeline's step list.","ref":"MDEx.Pipe.html#prepend_steps/2"},{"type":"function","title":"MDEx.Pipe.put_extension_options/2","doc":"Updates the pipeline's `:extension` options.","ref":"MDEx.Pipe.html#put_extension_options/2"},{"type":"function","title":"Examples - MDEx.Pipe.put_extension_options/2","doc":"iex> pipe = MDEx.Pipe.put_extension_options(MDEx.new(), table: true)\n    iex> MDEx.Pipe.get_option(pipe, :extension)[:table]\n    true","ref":"MDEx.Pipe.html#put_extension_options/2-examples"},{"type":"function","title":"MDEx.Pipe.put_node_in_document_root/3","doc":"Inserts `node` into the document root at the specified `position`.\n\n  - By default, the node is inserted at the top of the document.\n  - Node must be a valid fragment node like a `MDEx.Heading`, `MDEx.HtmlBlock`, etc.","ref":"MDEx.Pipe.html#put_node_in_document_root/3"},{"type":"function","title":"Examples - MDEx.Pipe.put_node_in_document_root/3","doc":"iex> pipe = MDEx.new()\n    iex> pipe = MDEx.Pipe.append_steps(\n    ...>   pipe,\n    ...>   append_node: fn pipe ->\n    ...>     html_block = %MDEx.HtmlBlock{literal: \" Hello \"}\n    ...>     MDEx.Pipe.put_node_in_document_root(pipe, html_block, :bottom)\n    ...>   end)\n    iex> MDEx.to_html(pipe, document: \"# Doc\", render: [unsafe: true])\n    {:ok, \" Doc \\n Hello \"}","ref":"MDEx.Pipe.html#put_node_in_document_root/3-examples"},{"type":"function","title":"MDEx.Pipe.put_options/2","doc":"Merges options into the pipeline's existing options.\n\nThis function handles both built-in options (`:document`, `:extension`, `:parse`, `:render`, `:syntax_highlight`, and `:sanitize`)\nand user-defined options that have been registered with `register_options/2`.","ref":"MDEx.Pipe.html#put_options/2"},{"type":"function","title":"Examples - MDEx.Pipe.put_options/2","doc":"iex> pipe = MDEx.Pipe.register_options(MDEx.new(), [:custom_option])\n    iex> pipe = MDEx.Pipe.put_options(pipe, [\n    ...>   document: \"# Hello\",\n    ...>   extension: [table: true],\n    ...>   custom_option: \"value\"\n    ...> ])\n    iex> MDEx.Pipe.get_option(pipe, :document)\n    \"# Hello\"\n    iex> MDEx.Pipe.get_option(pipe, :extension)[:table]\n    true\n    iex> MDEx.Pipe.get_option(pipe, :custom_option)\n    \"value\"\n\nBuilt-in options are validated against their respective schemas:\n\n    iex> try do\n    ...>   MDEx.Pipe.put_options(MDEx.new(), [extension: [invalid: true]])\n    ...> rescue\n    ...>   NimbleOptions.ValidationError -> :error\n    ...> end\n    :error","ref":"MDEx.Pipe.html#put_options/2-examples"},{"type":"function","title":"MDEx.Pipe.put_parse_options/2","doc":"Updates the pipeline's `:parse` options.","ref":"MDEx.Pipe.html#put_parse_options/2"},{"type":"function","title":"Examples - MDEx.Pipe.put_parse_options/2","doc":"iex> pipe = MDEx.Pipe.put_parse_options(MDEx.new(), smart: true)\n    iex> MDEx.Pipe.get_option(pipe, :parse)[:smart]\n    true","ref":"MDEx.Pipe.html#put_parse_options/2-examples"},{"type":"function","title":"MDEx.Pipe.put_private/3","doc":"Stores a value in the pipeline's private storage.","ref":"MDEx.Pipe.html#put_private/3"},{"type":"function","title":"MDEx.Pipe.put_render_options/2","doc":"Updates the pipeline's `:render` options.","ref":"MDEx.Pipe.html#put_render_options/2"},{"type":"function","title":"Examples - MDEx.Pipe.put_render_options/2","doc":"iex> pipe = MDEx.Pipe.put_render_options(MDEx.new(), escape: true)\n    iex> MDEx.Pipe.get_option(pipe, :render)[:escape]\n    true","ref":"MDEx.Pipe.html#put_render_options/2-examples"},{"type":"function","title":"MDEx.Pipe.put_sanitize_options/2","doc":"Updates the pipeline's `:sanitize` options.","ref":"MDEx.Pipe.html#put_sanitize_options/2"},{"type":"function","title":"Examples - MDEx.Pipe.put_sanitize_options/2","doc":"iex> pipe = MDEx.Pipe.put_sanitize_options(MDEx.new(), add_tags: [\"MyComponent\"])\n    iex> MDEx.Pipe.get_option(pipe, :sanitize)[:add_tags]\n    [\"MyComponent\"]","ref":"MDEx.Pipe.html#put_sanitize_options/2-examples"},{"type":"function","title":"MDEx.Pipe.put_syntax_highlight_options/2","doc":"Updates the pipeline's `:syntax_highlight` options.","ref":"MDEx.Pipe.html#put_syntax_highlight_options/2"},{"type":"function","title":"Examples - MDEx.Pipe.put_syntax_highlight_options/2","doc":"iex> pipe = MDEx.Pipe.put_syntax_highlight_options(MDEx.new(), formatter: :html_linked)\n    iex> MDEx.Pipe.get_option(pipe, :syntax_highlight)[:formatter]\n    :html_linked","ref":"MDEx.Pipe.html#put_syntax_highlight_options/2-examples"},{"type":"function","title":"MDEx.Pipe.register_options/2","doc":"Registers a list of valid options that can be used by steps in the pipeline.","ref":"MDEx.Pipe.html#register_options/2"},{"type":"function","title":"Examples - MDEx.Pipe.register_options/2","doc":"iex> pipe = MDEx.new()\n    iex> pipe = MDEx.Pipe.register_options(pipe, [:mermaid_version])\n    iex> pipe = MDEx.Pipe.put_options(pipe, mermaid_version: \"11\")\n    iex> pipe.options[:mermaid_version]\n    \"11\"\n\n    iex> MDEx.new(rendr: [unsafe: true])\n    ** (ArgumentError) unknown option :rendr. Did you mean :render?","ref":"MDEx.Pipe.html#register_options/2-examples"},{"type":"function","title":"MDEx.Pipe.run/1","doc":"Executes the pipeline steps in order.\n\nThis function is usually not called directly,\nprefer calling one of the `to_*` functions in `MDEx` module.","ref":"MDEx.Pipe.html#run/1"},{"type":"type","title":"MDEx.Pipe.step/0","doc":"Step in a pipeline.\n\nIt's a function that receives a `t:MDEx.Pipe.t/0` struct and must return either one of the following:\n\n  - a `t:MDEx.Pipe.t/0` struct\n  - a tuple with a `t:MDEx.Pipe.t/0` struct and an `t:Exception.t/0` as `{pipe, exception}`\n  - a tuple with a module, function and arguments which will be invoked with `apply/3`","ref":"MDEx.Pipe.html#t:step/0"},{"type":"type","title":"MDEx.Pipe.t/0","doc":"Pipeline state.","ref":"MDEx.Pipe.html#t:t/0"},{"type":"function","title":"MDEx.Pipe.update_nodes/3","doc":"Updates nodes in the document that match `selector`.","ref":"MDEx.Pipe.html#update_nodes/3"},{"type":"function","title":"MDEx.Pipe.update_private/4","doc":"Updates a value in the pipeline's private storage using a function.","ref":"MDEx.Pipe.html#update_private/4"},{"type":"module","title":"MDEx.Sigil","doc":"Sigils for parsing and formatting Markdown between different formats.","ref":"MDEx.Sigil.html"},{"type":"module","title":"Modifiers - MDEx.Sigil","doc":"* `HTML` - converts Markdown or `MDEx.Document` to HTML\n\n  Use [EEx.SmartEngine](https://hexdocs.pm/eex/EEx.SmartEngine.html) to the document into HTML. It does support `assigns` but only the old `<%= ... %>` syntax,\n  and it doesn't support components. It's useful if you want to generate static HTML from Markdown or don't need components or don't want to define an `assigns` variable (it's optional).\n\n  * `JSON` - converts Markdown or `MDEx.Document` to JSON\n\n  * `XML` - converts Markdown or `MDEx.Document` to XML\n\n  * `MD` - converts `MDEx.Document` to Markdown\n\n  * No modifier (default) - parses a Markdown string into a `MDEx.Document` struct\n\nNote that you should `import MDEx.Sigil` to use the `~MD` sigil.","ref":"MDEx.Sigil.html#module-modifiers"},{"type":"module","title":"HTML/EEx Format Order - MDEx.Sigil","doc":"In order to generate the final result, the Markdown string or `MDEx.Document` (initial input) is first converted into a static HTML without escaping\nthe content, then the HTML is passed to the appropriate engine to generate the final output.","ref":"MDEx.Sigil.html#module-html-eex-format-order"},{"type":"module","title":"Assigns and Expressions - MDEx.Sigil","doc":"Only the `HTML` modifier support assigns, any other modifier will render the assign unmodified.\nThat's particularly important when generating a `MDEx.Document` which does represent the Markdown AST because it must respect\nthe Markdown content and also be able to convert back to a Markdown string.\n\n> #### Expressions inside code blocks are preserved {: .warning}\n> Expressions as `<%= ... %>` or `{ ... }` inside code blocks are escaped and not evaluated, ie: they are preserved as is:\n> ```elixir\n> assigns = %{title: \"Hello\"}\n> ~MD\"\"\"\n`{@title}`\n> \"\"\"HTML\n> \"  &lbrace;@title&rbrace;  \"\n> ```","ref":"MDEx.Sigil.html#module-assigns-and-expressions"},{"type":"module","title":"Options - MDEx.Sigil","doc":"All modifiers use these options by default:\n\n```elixir\n[\n  extension: [\n    strikethrough: true,\n    table: true,\n    autolink: false,\n    tasklist: true,\n    superscript: true,\n    footnotes: true,\n    description_lists: true,\n    multiline_block_quotes: true,\n    alerts: true,\n    math_dollars: true,\n    math_code: true,\n    shortcodes: true,\n    underline: true,\n    spoiler: true\n  ],\n  parse: [relaxed_tasklist_matching: true, relaxed_autolinks: true],\n  render: [unsafe: true, escape: false]\n]\n```\n\nIf you need a different set of options, you can call the regular functions in `MDEx` to pass the options you need.","ref":"MDEx.Sigil.html#module-options"},{"type":"macro","title":"MDEx.Sigil.sigil_M/2","doc":"","ref":"MDEx.Sigil.html#sigil_M/2"},{"type":"macro","title":"MDEx.Sigil.sigil_MD/2","doc":"The `~MD` sigil converts a Markdown string or a `%MDEx.Document{}` struct to either one of these formats: `MDEx.Document`, Markdown (CommonMark), HTML, JSON or XML.","ref":"MDEx.Sigil.html#sigil_MD/2"},{"type":"macro","title":"Assigns - MDEx.Sigil.sigil_MD/2","doc":"You can define a variable `assigns` in the context of the sigil to evaluate expressions:\n\n    iex> assigns = %{lang: \":elixir\"}\n    iex> ~MD|`lang = <%= @lang %>`|HTML\n    \"  lang = :elixir  \"\n\nNote that only the `HTML` modifier support assigns.","ref":"MDEx.Sigil.html#sigil_MD/2-assigns"},{"type":"macro","title":"Examples - MDEx.Sigil.sigil_MD/2","doc":"","ref":"MDEx.Sigil.html#sigil_MD/2-examples"},{"type":"macro","title":"Markdown to MDEx.Document - MDEx.Sigil.sigil_MD/2","doc":"```elixir\niex> ~MD[`lang = :elixir`]\n%MDEx.Document{nodes: [%MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"lang = :elixir\"}]}]}\n```","ref":"MDEx.Sigil.html#sigil_MD/2-markdown-to-mdex-document"},{"type":"macro","title":"Markdown to HTML - MDEx.Sigil.sigil_MD/2","doc":"```elixir\niex> ~MD[`lang = :elixir`]HTML\n\"  lang = :elixir  \\n\"\n```","ref":"MDEx.Sigil.html#sigil_MD/2-markdown-to-html"},{"type":"macro","title":"Markdown to JSON - MDEx.Sigil.sigil_MD/2","doc":"```elixir\niex> ~MD[`lang = :elixir`]JSON\n\"{\"nodes\":[{\"nodes\":[{\"literal\":\"lang = :elixir\",\"num_backticks\":1,\"node_type\":\"MDEx.Code\"}],\"node_type\":\"MDEx.Paragraph\"}],\"node_type\":\"MDEx.Document\"}\"\n```","ref":"MDEx.Sigil.html#sigil_MD/2-markdown-to-json"},{"type":"macro","title":"Markdown to XML - MDEx.Sigil.sigil_MD/2","doc":"```elixir\niex> ~MD[`lang = :elixir`]XML\n\"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE document SYSTEM \"CommonMark.dtd\">\n \n   \n     lang = :elixir \n   \n \n\"\n```\n\n### `MDEx.Document` to Markdown\n\n```elixir\niex> ~MD|%MDEx.Document{nodes: [%MDEx.Paragraph{nodes: [%MDEx.Code{num_backticks: 1, literal: \"lang = :elixir\"}]}]}|MD\n\"`lang = :elixir`\"\n```","ref":"MDEx.Sigil.html#sigil_MD/2-markdown-to-xml"},{"type":"macro","title":"Elixir Expressions - MDEx.Sigil.sigil_MD/2","doc":"```elixir\niex> ~MD[","ref":"MDEx.Sigil.html#sigil_MD/2-elixir-expressions"},{"type":"macro","title":"Section <%= 1 + 1 %>]HTML - MDEx.Sigil.sigil_MD/2","doc":"\" Section 2 \"\n```","ref":"MDEx.Sigil.html#sigil_MD/2-section-1-1-html"},{"type":"macro","title":"MDEx.Sigil.sigil_m/2","doc":"","ref":"MDEx.Sigil.html#sigil_m/2"},{"type":"module","title":"MDEx.Alert","doc":"GitHub and GitLab style alerts / admonitions.\n\nSee [GitHub](https://docs.github.com/en/get-started/writing-on-github/getting-started-with-writing-and-formatting-on-github/basic-writing-and-formatting-syntax#alerts)\nand [GitLab](https://docs.gitlab.com/user/markdown/#alerts) docs.","ref":"MDEx.Alert.html"},{"type":"type","title":"MDEx.Alert.t/0","doc":"","ref":"MDEx.Alert.html#t:t/0"},{"type":"module","title":"MDEx.BlockQuote","doc":"A block quote marker.\n\nSpec: https://github.github.com/gfm/#block-quotes","ref":"MDEx.BlockQuote.html"},{"type":"type","title":"MDEx.BlockQuote.t/0","doc":"","ref":"MDEx.BlockQuote.html#t:t/0"},{"type":"module","title":"MDEx.Code","doc":"Inline code span.\n\nSpec: https://github.github.com/gfm/#code-spans","ref":"MDEx.Code.html"},{"type":"type","title":"MDEx.Code.t/0","doc":"","ref":"MDEx.Code.html#t:t/0"},{"type":"module","title":"MDEx.CodeBlock","doc":"A code block, fenced or indented.\n\nSpec: https://github.github.com/gfm/#fenced-code-blocks and https://github.github.com/gfm/#indented-code-blocks","ref":"MDEx.CodeBlock.html"},{"type":"type","title":"MDEx.CodeBlock.t/0","doc":"","ref":"MDEx.CodeBlock.html#t:t/0"},{"type":"module","title":"MDEx.DescriptionDetails","doc":"Description details of a description item.\n\nSee `MDEx.DescriptionList`","ref":"MDEx.DescriptionDetails.html"},{"type":"type","title":"MDEx.DescriptionDetails.t/0","doc":"","ref":"MDEx.DescriptionDetails.html#t:t/0"},{"type":"module","title":"MDEx.DescriptionItem","doc":"A description item of a description list.\n\nSee `MDEx.DescriptionList`","ref":"MDEx.DescriptionItem.html"},{"type":"type","title":"MDEx.DescriptionItem.t/0","doc":"","ref":"MDEx.DescriptionItem.html#t:t/0"},{"type":"module","title":"MDEx.DescriptionList","doc":"A description list.","ref":"MDEx.DescriptionList.html"},{"type":"type","title":"MDEx.DescriptionList.t/0","doc":"","ref":"MDEx.DescriptionList.html#t:t/0"},{"type":"module","title":"MDEx.DescriptionTerm","doc":"A description term of a description item.\n\nSee `MDEx.DescriptionList`","ref":"MDEx.DescriptionTerm.html"},{"type":"type","title":"MDEx.DescriptionTerm.t/0","doc":"","ref":"MDEx.DescriptionTerm.html#t:t/0"},{"type":"module","title":"MDEx.Emph","doc":"Emphasis.\n\nSpec: https://github.github.com/gfm/#emphasis-and-strong-emphasis","ref":"MDEx.Emph.html"},{"type":"type","title":"MDEx.Emph.t/0","doc":"","ref":"MDEx.Emph.html#t:t/0"},{"type":"module","title":"MDEx.Escaped","doc":"An escaped character.\n\nSpec: https://github.github.com/gfm/#backslash-escapes","ref":"MDEx.Escaped.html"},{"type":"type","title":"MDEx.Escaped.t/0","doc":"","ref":"MDEx.Escaped.html#t:t/0"},{"type":"module","title":"MDEx.EscapedTag","doc":"Escaped tag.","ref":"MDEx.EscapedTag.html"},{"type":"type","title":"MDEx.EscapedTag.t/0","doc":"","ref":"MDEx.EscapedTag.html#t:t/0"},{"type":"module","title":"MDEx.FootnoteDefinition","doc":"A footnote definition.","ref":"MDEx.FootnoteDefinition.html"},{"type":"type","title":"MDEx.FootnoteDefinition.t/0","doc":"","ref":"MDEx.FootnoteDefinition.html#t:t/0"},{"type":"module","title":"MDEx.FootnoteReference","doc":"The reference to a footnote.","ref":"MDEx.FootnoteReference.html"},{"type":"type","title":"MDEx.FootnoteReference.t/0","doc":"","ref":"MDEx.FootnoteReference.html#t:t/0"},{"type":"module","title":"MDEx.FrontMatter","doc":"Document metadata.","ref":"MDEx.FrontMatter.html"},{"type":"type","title":"MDEx.FrontMatter.t/0","doc":"","ref":"MDEx.FrontMatter.html#t:t/0"},{"type":"module","title":"MDEx.Heading","doc":"A heading, either ATX or setext.\n\nATX is the most common heading, a line starting with 1-6 `#` characters,\nand setext is represented as one or more lines followed by a heading underline as `===` or `---`.\n\nSpec: https://github.github.com/gfm/#atx-headings and https://github.github.com/gfm/#setext-headings","ref":"MDEx.Heading.html"},{"type":"type","title":"MDEx.Heading.t/0","doc":"","ref":"MDEx.Heading.html#t:t/0"},{"type":"module","title":"MDEx.HtmlBlock","doc":"A HTML block.\n\nSpec: https://github.github.com/gfm/#html-blocks","ref":"MDEx.HtmlBlock.html"},{"type":"type","title":"MDEx.HtmlBlock.t/0","doc":"","ref":"MDEx.HtmlBlock.html#t:t/0"},{"type":"module","title":"MDEx.HtmlInline","doc":"Raw HTML.\n\nSpec: https://github.github.com/gfm/#raw-html","ref":"MDEx.HtmlInline.html"},{"type":"type","title":"MDEx.HtmlInline.t/0","doc":"","ref":"MDEx.HtmlInline.html#t:t/0"},{"type":"module","title":"MDEx.Image","doc":"An image.\n\nSpec: https://github.github.com/gfm/#images","ref":"MDEx.Image.html"},{"type":"type","title":"MDEx.Image.t/0","doc":"","ref":"MDEx.Image.html#t:t/0"},{"type":"module","title":"MDEx.LineBreak","doc":"A hard line break.\n\nSpec: https://github.github.com/gfm/#hard-line-breaks","ref":"MDEx.LineBreak.html"},{"type":"type","title":"MDEx.LineBreak.t/0","doc":"","ref":"MDEx.LineBreak.html#t:t/0"},{"type":"module","title":"MDEx.Link","doc":"Link to a URL.\n\nSpec: https://github.github.com/gfm/#links","ref":"MDEx.Link.html"},{"type":"type","title":"MDEx.Link.t/0","doc":"","ref":"MDEx.Link.html#t:t/0"},{"type":"module","title":"MDEx.List","doc":"A List that contains `MDEx.ListItem`.\n\nSpec: https://github.github.com/gfm/#lists","ref":"MDEx.List.html"},{"type":"type","title":"MDEx.List.t/0","doc":"","ref":"MDEx.List.html#t:t/0"},{"type":"module","title":"MDEx.ListItem","doc":"A List Item of a `MDEx.List`.\n\nSpec: https://github.github.com/gfm/#list-items","ref":"MDEx.ListItem.html"},{"type":"type","title":"MDEx.ListItem.t/0","doc":"","ref":"MDEx.ListItem.html#t:t/0"},{"type":"module","title":"MDEx.Math","doc":"Inline math span.","ref":"MDEx.Math.html"},{"type":"type","title":"MDEx.Math.t/0","doc":"","ref":"MDEx.Math.html#t:t/0"},{"type":"module","title":"MDEx.MultilineBlockQuote","doc":"A multiline block quote.\n\nSpec: https://github.github.com/gfm/#block-quotes","ref":"MDEx.MultilineBlockQuote.html"},{"type":"type","title":"MDEx.MultilineBlockQuote.t/0","doc":"","ref":"MDEx.MultilineBlockQuote.html#t:t/0"},{"type":"module","title":"MDEx.Paragraph","doc":"A paragraph that contains nodes.\n\nSpec: https://github.github.com/gfm/#paragraphs","ref":"MDEx.Paragraph.html"},{"type":"type","title":"MDEx.Paragraph.t/0","doc":"","ref":"MDEx.Paragraph.html#t:t/0"},{"type":"module","title":"MDEx.Raw","doc":"A Raw output node. This will be inserted verbatim into CommonMark and HTML output. It can only be created programmatically, and is never parsed from input.","ref":"MDEx.Raw.html"},{"type":"type","title":"MDEx.Raw.t/0","doc":"","ref":"MDEx.Raw.html#t:t/0"},{"type":"module","title":"MDEx.ShortCode","doc":"Emoji generated from a shortcode.","ref":"MDEx.ShortCode.html"},{"type":"type","title":"MDEx.ShortCode.t/0","doc":"","ref":"MDEx.ShortCode.html#t:t/0"},{"type":"module","title":"MDEx.SoftBreak","doc":"A soft line break.\n\nSpec: https://github.github.com/gfm/#soft-line-breaks","ref":"MDEx.SoftBreak.html"},{"type":"type","title":"MDEx.SoftBreak.t/0","doc":"","ref":"MDEx.SoftBreak.html#t:t/0"},{"type":"module","title":"MDEx.SpoileredText","doc":"Spoilered text.","ref":"MDEx.SpoileredText.html"},{"type":"type","title":"MDEx.SpoileredText.t/0","doc":"","ref":"MDEx.SpoileredText.html#t:t/0"},{"type":"module","title":"MDEx.Strikethrough","doc":"Strikethrough.\n\nSpec: https://github.github.com/gfm/#strikethrough-extension-","ref":"MDEx.Strikethrough.html"},{"type":"type","title":"MDEx.Strikethrough.t/0","doc":"","ref":"MDEx.Strikethrough.html#t:t/0"},{"type":"module","title":"MDEx.Strong","doc":"Strong emphasis.\n\nSpec: https://github.github.com/gfm/#emphasis-and-strong-emphasis","ref":"MDEx.Strong.html"},{"type":"type","title":"MDEx.Strong.t/0","doc":"","ref":"MDEx.Strong.html#t:t/0"},{"type":"module","title":"MDEx.Subscript","doc":"Subscript.","ref":"MDEx.Subscript.html"},{"type":"type","title":"MDEx.Subscript.t/0","doc":"","ref":"MDEx.Subscript.html#t:t/0"},{"type":"module","title":"MDEx.Superscript","doc":"Superscript.","ref":"MDEx.Superscript.html"},{"type":"type","title":"MDEx.Superscript.t/0","doc":"","ref":"MDEx.Superscript.html#t:t/0"},{"type":"module","title":"MDEx.Table","doc":"A table with rows and columns.\n\nSpec: https://github.github.com/gfm/#tables-extension-","ref":"MDEx.Table.html"},{"type":"type","title":"MDEx.Table.t/0","doc":"","ref":"MDEx.Table.html#t:t/0"},{"type":"module","title":"MDEx.TableCell","doc":"A table cell inside a table row.\n\nSee `MDEx.Table`","ref":"MDEx.TableCell.html"},{"type":"type","title":"MDEx.TableCell.t/0","doc":"","ref":"MDEx.TableCell.html#t:t/0"},{"type":"module","title":"MDEx.TableRow","doc":"A table row.\n\nSee `MDEx.Table`","ref":"MDEx.TableRow.html"},{"type":"type","title":"MDEx.TableRow.t/0","doc":"","ref":"MDEx.TableRow.html#t:t/0"},{"type":"module","title":"MDEx.TaskItem","doc":"A task item inside a list.\n\nSpec: https://github.github.com/gfm/#task-list-items-extension-","ref":"MDEx.TaskItem.html"},{"type":"type","title":"MDEx.TaskItem.t/0","doc":"","ref":"MDEx.TaskItem.html#t:t/0"},{"type":"module","title":"MDEx.Text","doc":"Literal text.\n\nSpec: https://github.github.com/gfm/#textual-content","ref":"MDEx.Text.html"},{"type":"type","title":"MDEx.Text.t/0","doc":"","ref":"MDEx.Text.html#t:t/0"},{"type":"module","title":"MDEx.ThematicBreak","doc":"A break between lines.\n\nSpec: https://github.github.com/gfm/#thematic-breaks","ref":"MDEx.ThematicBreak.html"},{"type":"type","title":"MDEx.ThematicBreak.t/0","doc":"","ref":"MDEx.ThematicBreak.html#t:t/0"},{"type":"module","title":"MDEx.Underline","doc":"Underline.","ref":"MDEx.Underline.html"},{"type":"type","title":"MDEx.Underline.t/0","doc":"","ref":"MDEx.Underline.html#t:t/0"},{"type":"module","title":"MDEx.WikiLink","doc":"A link in the form of a wiki link.","ref":"MDEx.WikiLink.html"},{"type":"type","title":"MDEx.WikiLink.t/0","doc":"","ref":"MDEx.WikiLink.html#t:t/0"},{"type":"exception","title":"MDEx.DecodeError","doc":"Failed to decode a Document.\n\nUsually this means that a `MDEx.Document` is invalid and cannot be decoded.","ref":"MDEx.DecodeError.html"},{"type":"function","title":"MDEx.DecodeError.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"MDEx.DecodeError.html#message/1"},{"type":"type","title":"MDEx.DecodeError.t/0","doc":"","ref":"MDEx.DecodeError.html#t:t/0"},{"type":"exception","title":"MDEx.InvalidInputError","doc":"Given input is invalid.\n\nUsually this means that the input is not a string or a MDEx.Document struct.","ref":"MDEx.InvalidInputError.html"},{"type":"function","title":"MDEx.InvalidInputError.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"MDEx.InvalidInputError.html#message/1"},{"type":"type","title":"MDEx.InvalidInputError.t/0","doc":"","ref":"MDEx.InvalidInputError.html#t:t/0"},{"type":"exception","title":"MDEx.InvalidSelector","doc":"Invalid Access key selector.","ref":"MDEx.InvalidSelector.html"},{"type":"function","title":"MDEx.InvalidSelector.message/1","doc":"Callback implementation for  Exception.message/1 .","ref":"MDEx.InvalidSelector.html#message/1"},{"type":"type","title":"MDEx.InvalidSelector.t/0","doc":"","ref":"MDEx.InvalidSelector.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.7.4 (2025-07-02) - Changelog","doc":"","ref":"changelog.html#0-7-4-2025-07-02"},{"type":"extras","title":"Fixed - Changelog","doc":"- Accept the soft deprecated `:unsafe_` in `:render` options","ref":"changelog.html#fixed"},{"type":"extras","title":"0.7.3 (2025-06-24) - Changelog","doc":"","ref":"changelog.html#0-7-3-2025-06-24"},{"type":"extras","title":"Added - Changelog","doc":"- Add `MDEx.anchorize/1` to format text as an anchor (@paradox460)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Build `x86_64-pc-windows-msvc` target on `windows-2022` instead of unsupported `windows-2019`","ref":"changelog.html#changed"},{"type":"extras","title":"0.7.2 (2025-06-23) - Changelog","doc":"","ref":"changelog.html#0-7-2-2025-06-23"},{"type":"extras","title":"Fixed - Changelog","doc":"- Make sure `:syntax_highlight` is decoded with latest `autumn` version","ref":"changelog.html#fixed-1"},{"type":"extras","title":"0.7.1 (2025-06-20) - Changelog","doc":"","ref":"changelog.html#0-7-1-2025-06-20"},{"type":"extras","title":"Added - Changelog","doc":"- [Syntax Highlighter] Allow to disable the built-in syntax highlighter\n  - [Options] Limited support for `:image_url_rewriter` extension\n  - [Options] Limited support for `:link_url_rewriter` extension","ref":"changelog.html#added-1"},{"type":"extras","title":"Changed - Changelog","doc":"- [Sigil] Disable `autolink` to avoid conflicts on HEEx templates\n  - [Options] Rename `:unsafe_` to `:unsafe` in `:render` options. The old `:unsafe_` option still works.\n  - [Syntax Highlighter] Update `autumnus` to v0.4.0 with new languages and themes","ref":"changelog.html#changed-1"},{"type":"extras","title":"Fixed - Changelog","doc":"- [Sigils] Remove `smart: true` option to avoid automatic conversion of punctuation.\n  - [Docs] Update image in Dockerfile example","ref":"changelog.html#fixed-2"},{"type":"extras","title":"0.7.0 (2025-05-21) - Changelog","doc":"This versions introduces a new sigil `~MD` that supersedes the `~M` and `~m` sigils.\n\nTo migrate from `~M` to `~MD`, you can simply replace `~M` with `~MD` in your code.\n\nTo migrate from `~m` to `~MD`, you should define an `assigns` map with the values you want to expose:\n\n```elixir\n# before\nlang = \":elixir\"\n~m|`lang = #{lang}`|\n\n# after\nassigns = %{lang: \":elixir\"}\n~MD|`lang = <%= @lang %>`|\n```","ref":"changelog.html#0-7-0-2025-05-21"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Minimum required Elixir version is now v1.15","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Deprecate `~M` sigil in favor of `~MD`\n  * Deprecate `~m` sigil in favor of `~MD`","ref":"changelog.html#deprecations"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Introduce the `~MD` sigil","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.6.2 (2025-05-13) - Changelog","doc":"","ref":"changelog.html#0-6-2-2025-05-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Deps] Update comrak to v0.39\n  * [Deps] Move doc dependencies to `:doc` group to avoid unnecessary downloads\n  * [Sanitize] Added the following tag attribute rules by default:\n    - code: class translate tabindex\n    - pre: class style\n    - span: class style data-line\n  * [Pipe] Added `MDEx.Pipe.is_sanitize_enabled/1`\n  * [Pipe] Added `MDEx.Pipe.get_sanitize_option/3`","ref":"changelog.html#enhancements-1"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* [comrak] Removed option `:experimental_inline_sourcepos`. It's included by default now.","ref":"changelog.html#breaking-changes-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* [Syntax Highlight] Class and style are applied when sanitize is enabled","ref":"changelog.html#fixes"},{"type":"extras","title":"Docs - Changelog","doc":"* Add comparison table with other libraries","ref":"changelog.html#docs"},{"type":"extras","title":"0.6.1 (2025-04-18) - Changelog","doc":"","ref":"changelog.html#0-6-1-2025-04-18"},{"type":"extras","title":"Changes - Changelog","doc":"* [Syntax Highlighter] Defaults to `[formatter: [{:html_inline, theme: \"onedark\"}]]`","ref":"changelog.html#changes"},{"type":"extras","title":"0.6.0 (2025-04-18) - Changelog","doc":"This version introduces some minor breaking changes and some deprecations, see the change log below.\n\nThe biggest change is the migration from [Inkjet](https://crates.io/crates/inkjet) to [Autumnus](https://autumnus.dev) for the syntax highlighter,\nwhich can cause breaking changes in the syntax highlighter output, for example a missing theme or different class names or styles.\nPlease open an issue if you find any problems.","ref":"changelog.html#0-6-0-2025-04-18"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* [Syntax Highlighter] Renamed ` ` tag class from \"autumn-hl\" to \"athl\"\n  * [Syntax Highlighter] Changed ` ` tag class from \"ahl-{token}\" to \"{token}\", for eg: \"ahl-punctuation\" changed to \"punctuation\"\n  * Removed type `t:MDEx.features_option/0`\n  * Removed function `MDEx.default_features_options/0`\n  * Removed function `MDEx.Pipe.put_features_options/2`","ref":"changelog.html#breaking-changes-2"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Option `:features` is deprecated in favor of `:syntax_highlight` and `:sanitize`","ref":"changelog.html#deprecations-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Lines are now wrapped in ` ` tags as ` `, for eg: ` ...`\n  * Added `tabindex=\"0\"` attribute into ` ` tag for better accessibility\n  * Added function `MDEx.Pipe.put_syntax_highlight_options/2`\n  * Added function `MDEx.Pipe.put_sanitize_options/2`","ref":"changelog.html#enhancements-2"},{"type":"extras","title":"Changes - Changelog","doc":"* Replaced Inkjet with Autumnus in the Syntax Highlighter adapter","ref":"changelog.html#changes-1"},{"type":"extras","title":"0.5.0 (2025-03-31) - Changelog","doc":"","ref":"changelog.html#0-5-0-2025-03-31"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Introduce `MDEx.Pipe` - a Req-like pipeline to manipulate Markdown documents and write plugins\n  * Added `JSON` modifier to `~m` and `~M` sigils\n  * Added `MDEx.Document.wrap/1`","ref":"changelog.html#enhancements-3"},{"type":"extras","title":"Fixes - Changelog","doc":"* Revert to NIF 2.15 only","ref":"changelog.html#fixes-1"},{"type":"extras","title":"Changes - Changelog","doc":"* Updated `comrak` to v0.37.0","ref":"changelog.html#changes-2"},{"type":"extras","title":"0.4.3 (2025-03-29) - Changelog","doc":"","ref":"changelog.html#0-4-3-2025-03-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"* New logo\n  * Custom sanitization options by @kivikakk\n  * Added `to_json/1` and `to_json/2` to convert Markdown or MDEx.Document to JSON\n  * Added support to parse JSON to MDEx.Document in `parse_document/1` and `parse_document/2`\n  * Document and validate all options (comrak and ammonia)","ref":"changelog.html#enhancements-4"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix `:unsafe_` options in sigils, effectively enabling them\n  * [Docs] Tag `Alert` node as Document Node","ref":"changelog.html#fixes-2"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Bump minimum required Elixir version to 1.14\n  * Sigils now properly enable the `:unsafe_` option, which may cause breaking changes to some users","ref":"changelog.html#breaking-changes-3"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Renamed `to_commonmark/1` and `to_commonmark/2` to `to_markdown/1` and `to_markdown/2`\n  * Renamed `to_commonmark!/1` and `to_commonmark!/2` to `to_markdown!/1` and `to_markdown!/2`","ref":"changelog.html#deprecations-2"},{"type":"extras","title":"0.4.2 (2025-03-25) - Changelog","doc":"","ref":"changelog.html#0-4-2-2025-03-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added target `riscv64gc-unknown-linux-gnu target`\n  * Added binaries for NIF version 2.16","ref":"changelog.html#enhancements-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fixed glibc version mismatch on `x86_64-unknown-linux-gnu` target","ref":"changelog.html#fixes-3"},{"type":"extras","title":"Docs - Changelog","doc":"* Added section Pre-compilation listing all targets, how to compile the project,\n    and how to enable targets for legacy CPUs.\n  * Added example in `MDEx.Document` on how to bump Heading levels","ref":"changelog.html#docs-1"},{"type":"extras","title":"0.4.1 (2025-03-24) - Changelog","doc":"","ref":"changelog.html#0-4-1-2025-03-24"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added target `arm-unknown-linux-gnueabihf` used by Raspberry Pi","ref":"changelog.html#enhancements-6"},{"type":"extras","title":"0.4.0 (2025-03-10) - Changelog","doc":"","ref":"changelog.html#0-4-0-2025-03-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added support for [GitHub](https://docs.github.com/en/get-started/writing-on-github/getting-started-with-writing-and-formatting-on-github/basic-writing-and-formatting-syntax#alerts)\n    and [GitLab](https://docs.gitlab.com/user/markdown/#alerts) alerts.\n  * Process alerts by default in Sigils.\n  * Added `:experimental_minimize_commonmark` render option.","ref":"changelog.html#enhancements-7"},{"type":"extras","title":"Docs - Changelog","doc":"* Add nimble_publisher example by @PJUllrich","ref":"changelog.html#docs-2"},{"type":"extras","title":"Chores - Changelog","doc":"* Add sample Dockerfile for debugging","ref":"changelog.html#chores"},{"type":"extras","title":"0.3.3 (2025-02-03) - Changelog","doc":"","ref":"changelog.html#0-3-3-2025-02-03"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix `MDEx.Document.fetch/2` spec","ref":"changelog.html#fixes-4"},{"type":"extras","title":"0.3.2 (2025-01-11) - Changelog","doc":"","ref":"changelog.html#0-3-2-2025-01-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `MDEx.safe_html/2` utility function to sanitize and escape HTML content","ref":"changelog.html#enhancements-8"},{"type":"extras","title":"Fixes - Changelog","doc":"* HTML: encode `{` and `}` only inside ` ` tags to avoid disabling LiveView expressions","ref":"changelog.html#fixes-5"},{"type":"extras","title":"0.3.1 (2025-01-08) - Changelog","doc":"","ref":"changelog.html#0-3-1-2025-01-08"},{"type":"extras","title":"Enhancements - Changelog","doc":"* HTML: encode `{` and `}` as `&lbrace;` and `&rbrace;` to avoid LiveView syntax errors in HEEx templates","ref":"changelog.html#enhancements-9"},{"type":"extras","title":"0.3.0 (2024-12-16) - Changelog","doc":"Be aware, this version introduces major breaking changes:\n- AST format has changed\n- Sigils `~m` and `~M` now returns `%MDEx.Document{}` instead of a Markdown string (use the `MD` modifier to have the old behavior)\n\nThese changes enables the implementation of protocols to improve the manipulation of the AST,\nsee the [MDEx.Document](https://hexdocs.pm/mdex/MDEx.Document.html) module and [examples](https://github.com/leandrocp/mdex/tree/main/examples) for more info.","ref":"changelog.html#0-3-0-2024-12-16"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* Changed the AST format from `{name, attributes, children}` to structs as `%MDEx.Heading{level: 1, nodes: [%MDEx.Text{literal: \"Hello\"}]}`\n  * Sigils `~m` and `~M` now returns `%MDEx.Document{}` instead of a Markdown string\n  * Removed `MDEx.attribute/2` in favor of pattern matching key/value pairs in node structs","ref":"changelog.html#breaking-changes-4"},{"type":"extras","title":"Enhancements - Changelog","doc":"* New AST format based on structs\n  * Introduced modifiers `HTML`, `XML`, and `MD` for both sigils `~m` and `~M`\n  * Introduced `MDEx.traverse_and_update/3` with the `acc` argument\n  * Updated autumn to properly escape curly braces in HEEx template on LiveView 1.0\n  * Updated comrak to v0.31\n      - New node `MDEx.Subscript`\n      - New attr `is_task_item` in list nodes\n      - New option `render.figure_with_caption`\n      - New option `render.tasklist_classes`\n      - New option `render.ol_width`","ref":"changelog.html#enhancements-10"},{"type":"extras","title":"0.2.0 (2024-10-09) - Changelog","doc":"","ref":"changelog.html#0-2-0-2024-10-09"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* `to_html/1` and `to_html/2` now returns `{:ok, String.t()}` or `{:error, %MDEx.DecodeError{}}` instead of just `String.t()`.\n    The reason is because now they may accept an AST as input which may cause decoding errors.\n    Replace with `to_html!/1` and `to_html!/2` to have the same behavior as before.","ref":"changelog.html#breaking-changes-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix misspelling of `thematic` causing render errors - [#73](https://github.com/leandrocp/mdex/pull/73) by @jonklein","ref":"changelog.html#fixes-6"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `to_commonmark/1` and `to_commonmark/2` to convert an AST to CommonMark - [#70](https://github.com/leandrocp/mdex/pull/70) by @jonklein\n  * Added `~M` sigil (no interpolation) with `AST` and `MD` modifiers (defaults to HTML without the modifier)\n  * Added `~m` sigil (supports interpolation and escaping) with `AST` and `MD` modifiers (defaults to HTML without the modifier)\n  * Added `parse_document/1` and `parse_document/2` to parse Markdown to AST\n  * Added low-level functions `traverse_and_update/2` and `attribute/2` to manipulate AST\n  * Added `to_html!/1` and `to_html!/2`, the raising version of `to_html/1` and `to_html/2` (similar to previous `to_html/1` and `to_html/2`)\n  * Changed `to_html/1` and `to_html/2` to accept AST as input\n  * Added examples directory to show how to use the new APIs","ref":"changelog.html#enhancements-11"},{"type":"extras","title":"0.1.18 (2024-07-13) - Changelog","doc":"","ref":"changelog.html#0-1-18-2024-07-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Bump comrak from [0.24.1 to 0.26.0](https://github.com/kivikakk/comrak/blob/9f4d391abe2857031f993c4cdddf1ebba7cdbc7d/changelog.txt#L1-L60)\n  * Add new `extension` options: underline, spoiler, greentext\n  * Add new `render` options: experimental_inline_sourcepos, escaped_char_spans, ignore_setext, ignore_empty_links, gfm_quirks, prefer_fenced","ref":"changelog.html#enhancements-12"},{"type":"extras","title":"0.1.17 (2024-06-19) - Changelog","doc":"","ref":"changelog.html#0-1-17-2024-06-19"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Relax minimum required Elixir version to 1.13\n  * Bump comrak to 0.24.1\n  * Bump ammonia to 4.0\n  * Add new `extension` options: multiline_block_quotes, math_dollars, math_code, shortcodes, wikilinks_title_after_pipe, wikilinks_title_before_pipe\n  * Add new `render` option: escaped_char_spans\n  * Add new option `features.syntax_highlight_inline_style` to control whether to embed inline styles or not. Default is `true`.","ref":"changelog.html#enhancements-13"},{"type":"extras","title":"Changed - Changelog","doc":"* Build binaries on MacOS 12","ref":"changelog.html#changed-2"},{"type":"extras","title":"0.1.16 (2024-04-29) - Changelog","doc":"","ref":"changelog.html#0-1-16-2024-04-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added language `objc` to syntax highlighter.","ref":"changelog.html#enhancements-14"},{"type":"extras","title":"0.1.15 (2024-04-16) - Changelog","doc":"","ref":"changelog.html#0-1-15-2024-04-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update rustler to `~> 0.32`\n  * Update rustler_precompiled `~> 0.7`\n  * Added legacy targets","ref":"changelog.html#enhancements-15"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* Minimum required Elixir version is now 1.14\n  * Removed target `arm-unknown-linux-gnueabihf`\n  * Removed target `riscv64gc-unknown-linux-gnu`","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"0.1.14 (2024-04-11) - Changelog","doc":"","ref":"changelog.html#0-1-14-2024-04-11"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* [Syntax Highlight] Renamed parent ` ` class from `autumn-highlight` to `autumn-hl`\n  * [Syntax Highlight] Added prefix `ahl-` to each scope class","ref":"changelog.html#backwards-incompatible-changes-1"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update autumn to 0.2.2 (#33)\n  * Update comrak to 0.20.0 (#27) - @supernintendo","ref":"changelog.html#enhancements-16"},{"type":"extras","title":"0.1.13 (2023-11-20) - Changelog","doc":"","ref":"changelog.html#0-1-13-2023-11-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update autumn themes to add base16-tomorrow and base16-tomorrow-night - @paradox460","ref":"changelog.html#enhancements-17"},{"type":"extras","title":"0.1.12 (2023-11-06) - Changelog","doc":"","ref":"changelog.html#0-1-12-2023-11-06"},{"type":"extras","title":"Fixes - Changelog","doc":"* Comrak docs links","ref":"changelog.html#fixes-7"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add languages: jsx, tsx, vim\n  * Bump injket v0.10.2","ref":"changelog.html#enhancements-18"},{"type":"extras","title":"0.1.11 (2023-10-25) - Changelog","doc":"","ref":"changelog.html#0-1-11-2023-10-25"},{"type":"extras","title":"Fixes - Changelog","doc":"* Syntax highlighting - remove newlines to avoid formatting incorrectly","ref":"changelog.html#fixes-8"},{"type":"extras","title":"0.1.10 (2023-10-24) - Changelog","doc":"","ref":"changelog.html#0-1-10-2023-10-24"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add translate=\"no\" attr in ` ` tag","ref":"changelog.html#enhancements-19"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix Javascript syntax highlight\n  * Fix Typescript syntax highlight","ref":"changelog.html#fixes-9"},{"type":"extras","title":"0.1.9 (2023-09-29) - Changelog","doc":"","ref":"changelog.html#0-1-9-2023-09-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add logo\n  * Add icon","ref":"changelog.html#enhancements-20"},{"type":"extras","title":"0.1.8 (2023-09-29) - Changelog","doc":"","ref":"changelog.html#0-1-8-2023-09-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Fallback to plain text on invalid language\n  * Syntax highlight injections\n  * Add samples","ref":"changelog.html#enhancements-21"},{"type":"extras","title":"0.1.7 (2023-09-27) - Changelog","doc":"","ref":"changelog.html#0-1-7-2023-09-27"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Syntax highlight code using tree-sitter and helix editor themes. Use https://github.com/leandrocp/autumn/tree/main/native/inkjet_nif under the hood.","ref":"changelog.html#enhancements-22"},{"type":"extras","title":"0.1.6 (2023-09-14) - Changelog","doc":"","ref":"changelog.html#0-1-6-2023-09-14"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Load extra themes and syntaxes with https://crates.io/crates/two-face","ref":"changelog.html#enhancements-23"},{"type":"extras","title":"0.1.5 (2023-09-12) - Changelog","doc":"","ref":"changelog.html#0-1-5-2023-09-12"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Sanitize output with https://crates.io/crates/ammonia\n  * Syntax Highlight with https://crates.io/crates/syntect","ref":"changelog.html#enhancements-24"},{"type":"extras","title":"0.1.4 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-4-2023-09-11"},{"type":"extras","title":"Fixes - Changelog","doc":"* Compile on Ubuntu 20 to fix libc version mismatch","ref":"changelog.html#fixes-10"},{"type":"extras","title":"0.1.3 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-3-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* NIF version 2.15","ref":"changelog.html#enhancements-25"},{"type":"extras","title":"0.1.2 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-2-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Guard markdown arg","ref":"changelog.html#enhancements-26"},{"type":"extras","title":"Fixes - Changelog","doc":"* specs","ref":"changelog.html#fixes-11"},{"type":"extras","title":"0.1.1 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-1-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update Rust to edition 2021\n  * Add `@spec` to public functions","ref":"changelog.html#enhancements-27"},{"type":"extras","title":"0.1.0 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-0-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `MDEx.to_html/1` to convert Markdown to HTML using default options.\n  * `MDEx.to_html/2` to convert Markdown to HTML using custom options.","ref":"changelog.html#enhancements-28"},{"type":"extras","title":"Playground","doc":"# Playground\n\n```elixir\nMix.install([:req, :mdex])\n```","ref":"playground.html"},{"type":"extras","title":"HTML - Playground","doc":"```elixir\ndefmodule Playground do\n  @markdown Req.get!(\"https://raw.githubusercontent.com/leandrocp/mdex/refs/heads/main/README.md\").body\n\n  def run do\n    MDEx.to_html!(\n      @markdown,\n      extension: [\n        strikethrough: true,\n        tagfilter: true,\n        table: true,\n        autolink: true,\n        tasklist: true,\n        footnotes: true,\n        shortcodes: true,\n      ],\n      parse: [\n        smart: true,\n        relaxed_tasklist_matching: true,\n        relaxed_autolinks: true\n      ],\n      render: [\n        unsafe: true,\n      ],\n      syntax_highlight: [formatter: {:html_inline, theme: \"github_light\"}]\n    )\n  end\nend\n\nPlayground.run()\n|> IO.puts()\n```","ref":"playground.html#html"},{"type":"extras","title":"Comparison","doc":"# Comparison\n\n```elixir\nMix.install([\n  :mdex,\n  :earmark,\n  :md,\n  :cmark\n])\n```","ref":"comparison.html"},{"type":"extras","title":"Sample - Comparison","doc":"````elixir\nmd = ~s|\n# Sample","ref":"comparison.html#sample"},{"type":"extras","title":"Emoji - Comparison","doc":":rocket:","ref":"comparison.html#emoji"},{"type":"extras","title":"Links - Comparison","doc":"auto link: https://elixir-lang.org\n\nraw link:  Elixir","ref":"comparison.html#links"},{"type":"extras","title":"Code - Comparison","doc":"`self()`\n\n```elixir\ncurrent = self()\n```\n|\n````","ref":"comparison.html#code"},{"type":"extras","title":"MDEx - Comparison","doc":"```elixir\nmd\n|> MDEx.to_html!(render: [unsafe: true], extension: [shortcodes: true])\n|> IO.puts()\n```","ref":"comparison.html#mdex"},{"type":"extras","title":"Earmark - Comparison","doc":"```elixir\nmd\n|> Earmark.as_html!(gfm: true)\n|> IO.puts()\n```","ref":"comparison.html#earmark"},{"type":"extras","title":"md - Comparison","doc":"```elixir\nmd\n|> Md.generate()\n|> IO.puts()\n```","ref":"comparison.html#md"},{"type":"extras","title":"cmark - Comparison","doc":"```elixir\nmd\n|> Cmark.to_html([:unsafe])\n|> IO.puts()\n```","ref":"comparison.html#cmark"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}